//===========================================================================
//  @(#) $Name: arts++-0-9-b6 $
//  @(#) $Id: artsagg.cc,v 1.4 1999/02/04 19:55:54 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed of and
//  agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb,
//  and copyrighted(C) 1998 by the University of California,
//  San Diego (UCSD), with all rights reserved.  UCSD administers the
//  CAIDA grant, NCR-9711092, under which part of this code was
//  developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

extern "C" {
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <time.h>
#include <errno.h>
}

#include <fstream.h>
#include <string>
#include <map>

#include "ArtsProtocolTableAggregatorMap.hh"
#include "ArtsInterfaceMatrixAggregatorMap.hh"
#include "ArtsNextHopTableAggregatorMap.hh"
#include "ArtsAsMatrixAggregatorMap.hh"
#include "ArtsNetMatrixAggregatorMap.hh"
#include "ArtsPortMatrixAggregatorMap.hh"
#include "Arts.hh"
#include "ArtsDebug.hh"
#include "ArtsPackageVersion.hh"

extern char *tzname[2];
extern int   errno;

const string rcsid = "@(#) $Name: arts++-0-9-b6 $ $Id: artsagg.cc,v 1.4 1999/02/04 19:55:54 dwm Exp $";
static const ArtsPackageVersion  k_artsVersion = ArtsPackageVersion(rcsid);

static ArtsProtocolTableAggregatorMap    g_protoAggregatorMap;
static ArtsInterfaceMatrixAggregatorMap  g_intfmAggregatorMap;
static ArtsNextHopTableAggregatorMap     g_nextHopAggregatorMap;
static ArtsAsMatrixAggregatorMap         g_asmAggregatorMap;
static ArtsNetMatrixAggregatorMap        g_netmAggregatorMap;
static ArtsPortMatrixAggregatorMap       g_portmAggregatorMap;
static float                             g_hours = -1.0;

//-------------------------------------------------------------------------
//                      void Usage(const char *argv0)                      
//.........................................................................
//  
//-------------------------------------------------------------------------
void Usage(const char *argv0)
{
  cerr << "Usage: " << argv0 << " [-v] [-h hours] [-o] [-q] outfile infile(s)"
       << endl;
  return;
}

//----------------------------------------------------------------------------
//  static void FinishProtocolTableAggregation(ofstream & out, bool quiet) 
//............................................................................
//  
//----------------------------------------------------------------------------
static void FinishProtocolTableAggregation(ofstream & out, bool quiet)
{
  ArtsProtocolTableAggregatorMap::iterator  protoAggIter;
  ArtsProtocolTable  *protocolTable = (ArtsProtocolTable *)0;

  if ((g_protoAggregatorMap).size() > 0) {
    for (protoAggIter = g_protoAggregatorMap.begin();
         protoAggIter != g_protoAggregatorMap.end(); protoAggIter++) {
      protocolTable =
        ((*protoAggIter).second)->ConvertToArtsProtocolTable();
      protocolTable->write(out);
      delete(protocolTable);
      delete((*protoAggIter).second);
      if (!quiet) {
        cout << "+";
        cout.flush();
      }
    }
    g_protoAggregatorMap.erase(g_protoAggregatorMap.begin(),
                               g_protoAggregatorMap.end());
  }
  return;
}

//----------------------------------------------------------------------------
// static void FinishInterfaceMatrixAggregation(ofstream & out, bool quiet) 
//............................................................................
//  
//----------------------------------------------------------------------------
static void FinishInterfaceMatrixAggregation(ofstream & out, bool quiet)
{
  ArtsInterfaceMatrixAggregatorMap::iterator    intfmAggIter;
  ArtsInterfaceMatrix       *intfMatrix = (ArtsInterfaceMatrix *)NULL;

  if (g_intfmAggregatorMap.size() > 0) {
    for (intfmAggIter = g_intfmAggregatorMap.begin();
         intfmAggIter != g_intfmAggregatorMap.end(); intfmAggIter++) {
      intfMatrix = ((*intfmAggIter).second)->ConvertToArtsInterfaceMatrix();
      intfMatrix->write(out);
      delete(intfMatrix);
      delete((*intfmAggIter).second);
      if (!quiet) {
        cout << "+";
        cout.flush();
      }
    }
    g_intfmAggregatorMap.erase(g_intfmAggregatorMap.begin(),
                               g_intfmAggregatorMap.end());
  }
  return;
}

//----------------------------------------------------------------------------
//  static void FinishNextHopTableAggregation(ofstream & out, bool quiet)  
//............................................................................
//  
//----------------------------------------------------------------------------
static void FinishNextHopTableAggregation(ofstream & out, bool quiet)
{
  ArtsNextHopTableAggregatorMap::iterator   nextHopAggIter;
  ArtsNextHopTable  *nextHopTable = (ArtsNextHopTable *)NULL;

  if (g_nextHopAggregatorMap.size() > 0) {
    for (nextHopAggIter = g_nextHopAggregatorMap.begin();
         nextHopAggIter != g_nextHopAggregatorMap.end(); nextHopAggIter++) {
      nextHopTable =
        ((*nextHopAggIter).second)->ConvertToArtsNextHopTable();
      nextHopTable->write(out);
      delete(nextHopTable);
      delete((*nextHopAggIter).second);
      if (!quiet) {
        cout << "+";
        cout.flush();
      }
    }
    g_nextHopAggregatorMap.erase(g_nextHopAggregatorMap.begin(),
                                 g_nextHopAggregatorMap.end());
  }
  return;
}

//----------------------------------------------------------------------------
//    static void FinishAsMatrixAggregation(ofstream & out, bool quiet)    
//............................................................................
//  
//----------------------------------------------------------------------------
static void FinishAsMatrixAggregation(ofstream & out, bool quiet)
{
  ArtsAsMatrixAggregatorMap::iterator     asmAggIter;
  ArtsAsMatrix                           *asMatrix = (ArtsAsMatrix *)NULL;
  
  if (g_asmAggregatorMap.size() > 0) {
    for (asmAggIter = g_asmAggregatorMap.begin();
         asmAggIter != g_asmAggregatorMap.end(); asmAggIter++) {
      asMatrix = ((*asmAggIter).second)->ConvertToArtsAsMatrix();
      asMatrix->write(out);
      delete(asMatrix);
      delete((*asmAggIter).second);
      if (!quiet) {
        cout << "+";
        cout.flush();
      }
    }
    g_asmAggregatorMap.erase(g_asmAggregatorMap.begin(),
                             g_asmAggregatorMap.end());
    
  }
  return;
}

//----------------------------------------------------------------------------
//    static void FinishNetMatrixAggregation(ofstream & out, bool quiet)   
//............................................................................
//  
//----------------------------------------------------------------------------
static void FinishNetMatrixAggregation(ofstream & out, bool quiet)
{
  ArtsNetMatrixAggregatorMap::iterator    netmAggIter;
  ArtsNetMatrix                          *netMatrix = (ArtsNetMatrix *)NULL;

  if (g_netmAggregatorMap.size() > 0) {
    for (netmAggIter = g_netmAggregatorMap.begin();
         netmAggIter != g_netmAggregatorMap.end(); netmAggIter++) {
      netMatrix = ((*netmAggIter).second)->ConvertToArtsNetMatrix();
      netMatrix->write(out);
      delete(netMatrix);
      delete((*netmAggIter).second);
      if (!quiet) {
        cout << "+";
        cout.flush();
      }
    }

    g_netmAggregatorMap.erase(g_netmAggregatorMap.begin(),
                              g_netmAggregatorMap.end());
  }
  return;
}

//----------------------------------------------------------------------------
//   static void FinishPortMatrixAggregation(ofstream & out, bool quiet)   
//............................................................................
//  
//----------------------------------------------------------------------------
static void FinishPortMatrixAggregation(ofstream & out, bool quiet)
{
  ArtsPortMatrixAggregatorMap::iterator   portmAggIter;
  ArtsPortMatrix *portMatrix = (ArtsPortMatrix *)NULL;

  if (g_portmAggregatorMap.size() > 0) {
    for (portmAggIter = g_portmAggregatorMap.begin();
         portmAggIter != g_portmAggregatorMap.end(); portmAggIter++) {
      portMatrix = ((*portmAggIter).second)->ConvertToArtsPortMatrix();
      portMatrix->write(out);
      delete(portMatrix);
      delete((*portmAggIter).second);
      if (!quiet) {
        cout << "+";
        cout.flush();
      }
    }
    g_portmAggregatorMap.erase(g_portmAggregatorMap.begin(),
                               g_portmAggregatorMap.end());
  }
  return;
}

//----------------------------------------------------------------------------
//    static void                               
//    AggregateProtocolTableData(Arts * arts, ofstream & out, bool quiet) 
//............................................................................
//  
//----------------------------------------------------------------------------
static void
AggregateProtocolTableData(Arts * arts, ofstream & out, bool quiet)
{
  vector<ArtsAttribute>::const_iterator       periodAttribute;
  ArtsProtocolTableAggregatorMap::iterator    protoAggIter;
  static time_t                               intervalStart = 0;
  ArtsProtocolTable      *protocolTable = (ArtsProtocolTable *)NULL;
  
  g_protoAggregatorMap.Add(*arts);
  periodAttribute = arts->FindPeriodAttribute();
  if (intervalStart == 0) {
    intervalStart = ((*periodAttribute).Period())[0];
  }

  if (g_hours > 0.0) {
    if ((periodAttribute->Period())[1] >
        (intervalStart + g_hours * 60 * 60)) {
      for (protoAggIter = g_protoAggregatorMap.begin();
           protoAggIter != g_protoAggregatorMap.end(); protoAggIter++) {
        protocolTable =
          ((*protoAggIter).second)->ConvertToArtsProtocolTable();
        protocolTable->write(out);
        delete(protocolTable);
        delete((*protoAggIter).second);
        intervalStart = 0;
        if (!quiet) {
          cout << "+";
          cout.flush();
        }
      }
      g_protoAggregatorMap.erase(g_protoAggregatorMap.begin(),
                                 g_protoAggregatorMap.end());
    }
  }
  return;
}

//----------------------------------------------------------------------------
//   static void                               
//   AggregateInterfaceMatrixData(Arts * arts, ofstream & out, bool quiet) 
//............................................................................
//  
//----------------------------------------------------------------------------
static void
AggregateInterfaceMatrixData(Arts * arts, ofstream & out, bool quiet)
{
  vector<ArtsAttribute>::const_iterator       periodAttribute;
  ArtsInterfaceMatrixAggregatorMap::iterator  intfmAggIter;
  static time_t                               intervalStart = 0;
  ArtsInterfaceMatrix         *intfMatrix = (ArtsInterfaceMatrix *)NULL;

  //  Add the data to the aggregator map.
  g_intfmAggregatorMap.Add(*arts);

  periodAttribute = arts->FindPeriodAttribute();
  if (intervalStart == 0) {
    intervalStart = ((*periodAttribute).Period())[0];
  }

  if (g_hours > 0.0) {
    if ((periodAttribute->Period())[1] >
        (intervalStart + g_hours * 60 * 60)) {
      
      for (intfmAggIter = g_intfmAggregatorMap.begin();
           intfmAggIter != g_intfmAggregatorMap.end(); intfmAggIter++) {
        intfMatrix =
          ((*intfmAggIter).second)->ConvertToArtsInterfaceMatrix();
        intfMatrix->write(out);
        delete(intfMatrix);
        delete((*intfmAggIter).second);
        intervalStart = 0;
        if (!quiet) {
          cout << "+";
          cout.flush();
        }
      }
      
      g_intfmAggregatorMap.erase(g_intfmAggregatorMap.begin(),
                                 g_intfmAggregatorMap.end());
    }
  }
  return;
}

//----------------------------------------------------------------------------
//      static void                               
//      AggregateNextHopTable(Arts * arts, ofstream & out, bool quiet) 
//............................................................................
//  
//----------------------------------------------------------------------------
static void
AggregateNextHopTable(Arts * arts, ofstream & out, bool quiet)
{
  vector<ArtsAttribute>::const_iterator   periodAttribute;
  static time_t                           intervalStart = 0;
  ArtsNextHopTableAggregatorMap::iterator nextHopAggIter;
  ArtsNextHopTable      *nextHopTable = (ArtsNextHopTable *)NULL;

  g_nextHopAggregatorMap.Add(*arts);
  periodAttribute = arts->FindPeriodAttribute();
  if (intervalStart == 0) {
    intervalStart = ((*periodAttribute).Period())[0];
  }

  if (g_hours > 0.0) {
    if ((periodAttribute->Period())[1] >
        (intervalStart + g_hours * 60 * 60)) {
      for (nextHopAggIter = g_nextHopAggregatorMap.begin();
           nextHopAggIter != g_nextHopAggregatorMap.end();
           nextHopAggIter++) {
        nextHopTable =
          ((*nextHopAggIter).second)->ConvertToArtsNextHopTable();
        nextHopTable->write(out);
        delete(nextHopTable);
        delete((*nextHopAggIter).second);
        intervalStart = 0;
        if (!quiet) {
          cout << "+";
          cout.flush();
        }
      }
      g_nextHopAggregatorMap.erase(g_nextHopAggregatorMap.begin(),
                                   g_nextHopAggregatorMap.end());
    }
  }
  return;
}

//----------------------------------------------------------------------------
//       static void                               
//       AggregateAsMatrixData(Arts * arts, ofstream & out, bool quiet) 
//............................................................................
//  
//----------------------------------------------------------------------------
static void
AggregateAsMatrixData(Arts * arts, ofstream & out, bool quiet)
{
  vector<ArtsAttribute>::const_iterator   periodAttribute;
  static time_t                           intervalStart = 0;
  ArtsAsMatrixAggregatorMap::iterator     asmAggIter;
  ArtsAsMatrix                           *asMatrix = (ArtsAsMatrix *)NULL;
  
  //  Add the data to the aggregator map.
  g_asmAggregatorMap.Add(*arts);
  
  periodAttribute = arts->FindPeriodAttribute();
  if (intervalStart == 0) {
    intervalStart = ((*periodAttribute).Period())[0];
  }
  
  if (g_hours > 0.0) {
    if ((periodAttribute->Period())[1] >
        (intervalStart + g_hours * 60 * 60)) {
      for (asmAggIter = g_asmAggregatorMap.begin();
           asmAggIter != g_asmAggregatorMap.end(); asmAggIter++) {
        asMatrix = ((*asmAggIter).second)->ConvertToArtsAsMatrix();
        asMatrix->write(out);
        delete(asMatrix);
        delete((*asmAggIter).second);
        intervalStart = 0;
        if (!quiet) {
          cout << "+";
          cout.flush();
        }
      }
      g_asmAggregatorMap.erase(g_asmAggregatorMap.begin(),
                               g_asmAggregatorMap.end());
    }
  }
  return;
}

//----------------------------------------------------------------------------
//      static void                               
//      AggregateNetMatrixData(Arts * arts, ofstream & out, bool quiet) 
//............................................................................
//  
//----------------------------------------------------------------------------
static void
AggregateNetMatrixData(Arts * arts, ofstream & out, bool quiet)
{
  vector<ArtsAttribute>::const_iterator   periodAttribute;
  static time_t                           intervalStart = 0;
  ArtsNetMatrixAggregatorMap::iterator    netmAggIter;
  ArtsNetMatrix                          *netMatrix = (ArtsNetMatrix *)NULL;
  
  g_netmAggregatorMap.Add(*arts);

  periodAttribute = arts->FindPeriodAttribute();

  if (intervalStart == 0) {
    intervalStart = ((*periodAttribute).Period())[0];
  }

  if (g_hours > 0.0) {
    if ((periodAttribute->Period())[1] >
        (intervalStart + g_hours * 60 * 60)) {
      for (netmAggIter = g_netmAggregatorMap.begin();
           netmAggIter != g_netmAggregatorMap.end(); netmAggIter++) {
        netMatrix = ((*netmAggIter).second)->ConvertToArtsNetMatrix();
        netMatrix->write(out);
        delete(netMatrix);
        delete((*netmAggIter).second);
        intervalStart = 0;
        if (!quiet) {
          cout << "+";
          cout.flush();
        }
      }
      g_netmAggregatorMap.erase(g_netmAggregatorMap.begin(),
                                g_netmAggregatorMap.end());
    }
  }
  return;
}

//----------------------------------------------------------------------------
//     static void                               
//     AggregatePortMatrixData(Arts * arts, ofstream & out, bool quiet) 
//............................................................................
//  
//----------------------------------------------------------------------------
static void
AggregatePortMatrixData(Arts * arts, ofstream & out, bool quiet)
{
  vector<ArtsAttribute>::const_iterator   periodAttribute;
  static time_t                           intervalStart = 0;
  ArtsPortMatrixAggregatorMap::iterator   portmAggIter;
  ArtsPortMatrix      *portMatrix = (ArtsPortMatrix *)NULL;
  
  g_portmAggregatorMap.Add(*arts);
  periodAttribute = arts->FindPeriodAttribute();

  if (intervalStart == 0) {
    intervalStart = ((*periodAttribute).Period())[0];
  }

  if (g_hours > 0.0) {
    if ((periodAttribute->Period())[1] >
        (intervalStart + g_hours * 60 * 60)) {
      for (portmAggIter = g_portmAggregatorMap.begin();
           portmAggIter != g_portmAggregatorMap.end();
           portmAggIter++) {
        //  generate an ArtsPortMatrix object and write it.
        portMatrix =
          ((*portmAggIter).second)->ConvertToArtsPortMatrix();
        portMatrix->write(out);
        delete(portMatrix);
        delete((*portmAggIter).second);
        intervalStart = 0;
        if (!quiet) {
          cout << "+";
          cout.flush();
        }
      }
      g_portmAggregatorMap.erase(g_portmAggregatorMap.begin(),
                                 g_portmAggregatorMap.end());
    }
  }
  return;
}
                                      
//-------------------------------------------------------------------------
//                     int main(int argc, char *argv[])                    
//.........................................................................
//  
//-------------------------------------------------------------------------
int main(int argc, char *argv[])
{
  Arts        *arts;
  ArtsHeader  *artsHeader = (ArtsHeader *)0;
  ifstream    *artsStream;
  ofstream    *out;
  extern char *optarg;
  extern int   optind;
  int          optChar;
  bool         quiet = false;
  bool         overwrite = false;

  artsHeader = new ArtsHeader();
  if (artsHeader == (ArtsHeader *)0) {
    cerr << "failed to allocate new ArtsHeader." << endl;
    exit(1);
  }
    
  while ((optChar = getopt(argc,argv,"h:oqv")) != EOF) {
    switch (optChar) {
      case 'h':
        g_hours = atof(optarg);
        break;
      case 'o':
        overwrite = true;
        break;
      case 'q':
        quiet = true;
        break;
      case 'v':
        cerr << k_artsVersion.Name() << "  " << k_artsVersion.Id() << endl;
        exit(0);
        break;
      case '?':
      default:
        Usage(argv[0]);
        exit(1);
        break;
    }
  }

  if (optind >= (argc-1)) {
    Usage(argv[0]);
    exit(1);
  }

  if (overwrite)
    out = new ofstream(argv[optind],ios::out|ios::trunc);
  else
    out = new ofstream(argv[optind],ios::out|ios::app);

  if (!out || !(*out)) {
    cerr << "[A] unable to open '" << argv[optind] << "' as output file: "
         << strerror(errno) << endl;
    Usage(argv[0]);
    exit(1);
  }
  
  optind++;

  streampos  streamPosition;

  for (int fileNum = optind; fileNum < argc; fileNum++) {
    artsStream = new ifstream(argv[fileNum]);
    if (!artsStream || !(*artsStream)) {
      cerr << "[E] unable to open input file '" << argv[fileNum] << "': "
           << strerror(errno) << endl;
      continue;
    }
    else {
      while (*artsStream) {
        //  save our stream position, we might want to back up
        //  and read a whole ARTS object.
        streamPosition = artsStream->tellg();

        //  Read the object header from the stream.  We'll use it to
        //  determine whether or not the object is an AS matrix
        //  object.
        artsHeader->read(*artsStream);
        if (! *artsStream)
          break;

        switch (artsHeader->Identifier()) {
          case artsC_OBJECT_PROTO:
            //  It's a protocol table object.  Back up to start of header
            //  and read the whole object.
            artsStream->seekg(streamPosition,ios::beg);
            arts = new Arts();
            arts->read(*artsStream);
            if (!quiet) {
              cout << ".";
              cout.flush();
            }

            //  aggregate protocol table.
            AggregateProtocolTableData(arts,*out,quiet);

            delete(arts);
            break;
            
          case artsC_OBJECT_INTERFACE_MATRIX:
            //  It's an interface matrix object.  Back up to start of header
            //  and read the whole object.
            artsStream->seekg(streamPosition,ios::beg);
            arts = new Arts();
            arts->read(*artsStream);

            if (!quiet) {
              cout << ".";
              cout.flush();
            }

            //  aggregate interface matrix.
            AggregateInterfaceMatrixData(arts,*out,quiet);

            delete(arts);
            break;

          case artsC_OBJECT_NEXT_HOP:
            //  It's a NextHop table object.  Back up to start of header
            //  and read the whole object.
            artsStream->seekg(streamPosition,ios::beg);
            arts = new Arts();
            arts->read(*artsStream);

            if (!quiet) {
              cout << ".";
              cout.flush();
            }

            //  aggregate nexthop table.
            AggregateNextHopTable(arts,*out,quiet);

            delete(arts);
            break;
            
          case artsC_OBJECT_AS_MATRIX:
            //  It's an AS matrix object.  Back up to start of header
            //  and read the whole object.
            artsStream->seekg(streamPosition,ios::beg);
            arts = new Arts();
            arts->read(*artsStream);
            if (!quiet) {
              cout << ".";
              cout.flush();
            }

            //  aggregate the AS matrix.
            AggregateAsMatrixData(arts,*out,quiet);

            delete(arts);
            break;

          case artsC_OBJECT_NET:
            //  It's a net matrix object.  Back up to start of header
            //  and read the whole object.
            artsStream->seekg(streamPosition,ios::beg);
            arts = new Arts();
            arts->read(*artsStream);

            if (!quiet) {
              cout << ".";
              cout.flush();
            }

            //  aggregate the net matrix.
            AggregateNetMatrixData(arts,*out,quiet);

            delete(arts);
            break;

          case artsC_OBJECT_PORT_MATRIX:
            //  It's a port matrix object.  Back up to start of header
            //  and read the whole object.
            artsStream->seekg(streamPosition,ios::beg);
            arts = new Arts();
            arts->read(*artsStream);

            if (!quiet) {
              cout << ".";
              cout.flush();
            }
            AggregatePortMatrixData(arts,*out,quiet);

            delete(arts);
            break;
            
          default:
            //  Not an interesting object, skip over it.
            artsStream->seekg(artsHeader->AttrLength() +
                              artsHeader->DataLength(),ios::cur);
        }
      }
      artsStream->close();
    }
    delete(artsStream);
  }

  delete(artsHeader);
  
  FinishProtocolTableAggregation(*out,quiet);
  FinishInterfaceMatrixAggregation(*out,quiet);
  FinishNextHopTableAggregation(*out,quiet);
  FinishAsMatrixAggregation(*out,quiet);
  FinishNetMatrixAggregation(*out,quiet);
  FinishPortMatrixAggregation(*out,quiet);

  if (!quiet)
    cout << endl;
  
  out->close();
  delete(out);
  
  #ifdef ARTS_DEBUG_CLASS_ALLOC
    atexit(ShowArtsObjectsNotFreed);
  #endif
    
  exit(0);
}
