//===========================================================================
//  @(#) $Name: arts++-0-9-b6 $
//  @(#) $Id: artsasagg.cc,v 1.14 1999/02/04 19:56:23 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed of and
//  agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb,
//  and copyrighted(C) 1998 by the University of California,
//  San Diego (UCSD), with all rights reserved.  UCSD administers the
//  CAIDA grant, NCR-9711092, under which part of this code was
//  developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

extern "C" {
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <time.h>
#include <errno.h>
}

#include <fstream.h>
#include <string>
#include <map>

#include "ArtsAsMatrixAggregatorMap.hh"
#include "Arts.hh"
#include "ArtsPackageVersion.hh"
#ifdef ARTS_DEBUG_CLASS_ALLOC
  #include "ArtsDebug.hh"
#endif

extern char *tzname[2];
extern int   errno;

const string rcsid = "@(#) $Name: arts++-0-9-b6 $ $Id: artsasagg.cc,v 1.14 1999/02/04 19:56:23 dwm Exp $";
static const ArtsPackageVersion  k_artsVersion = ArtsPackageVersion(rcsid);

static ArtsAsMatrixAggregatorMap  g_asmAggregatorMap;

//-------------------------------------------------------------------------
//                      void Usage(const char *argv0)                      
//.........................................................................
//  
//-------------------------------------------------------------------------
void Usage(const char *argv0)
{
  cerr << "Usage: " << argv0 << " [-v] [-h hours] [-o] [-q] outfile infile(s)"
       << endl;
  return;
}

//-------------------------------------------------------------------------
//                     int main(int argc, char *argv[])                    
//.........................................................................
//  
//-------------------------------------------------------------------------
int main(int argc, char *argv[])
{
  Arts        *arts;
  ArtsHeader   artsHeader;
  ifstream    *artsStream;
  ofstream    *out;
  extern char *optarg;
  extern int   optind;
  int          optChar;
  float        hours = -1.0;
  time_t       intervalStart = 0;
  bool         quiet = false;
  bool         overwrite = false;
  vector<ArtsAttribute>::const_iterator   periodAttribute;
  ArtsAsMatrixAggregatorMap::iterator     asmAggIter;

  while ((optChar = getopt(argc,argv,"h:oqv")) != EOF) {
    switch (optChar) {
      case 'h':
        hours = atof(optarg);
        break;
      case 'o':
        overwrite = true;
        break;
      case 'q':
        quiet = true;
        break;
      case 'v':
        cerr << k_artsVersion.Name() << "  " << k_artsVersion.Id() << endl;
        exit(0);
        break;
      case '?':
      default:
        Usage(argv[0]);
        exit(1);
        break;
    }
  }

  if (optind >= (argc-1)) {
    Usage(argv[0]);
    exit(1);
  }

  if (overwrite)
    out = new ofstream(argv[optind],ios::out|ios::trunc);
  else
    out = new ofstream(argv[optind],ios::out|ios::app);

  if (!out || !(*out)) {
    cerr << "[A] unable to open '" << argv[optind] << "' as output file: "
         << strerror(errno) << endl;
    Usage(argv[0]);
    exit(1);
  }
  
  optind++;

  streampos  streamPosition;
      
  ArtsAsMatrix *asMatrix = (ArtsAsMatrix *)NULL;
  
  for (int fileNum = optind; fileNum < argc; fileNum++) {
    artsStream = new ifstream(argv[fileNum]);
    if (!artsStream || !(*artsStream)) {
      cerr << "[E] unable to open input file '" << argv[fileNum] << "': "
           << strerror(errno) << endl;
      continue;
    }
    else {
      while (*artsStream) {
        //  save our stream position, we might want to back up
        //  and read a whole ARTS object.
        streamPosition = artsStream->tellg();

        //  Read the object header from the stream.  We'll use it to
        //  determine whether or not the object is an AS matrix
        //  object.
        artsHeader.read(*artsStream);
        if (! *artsStream)
          break;
        
        if (artsHeader.Identifier() != artsC_OBJECT_AS_MATRIX) {
          //  Not an AS matrix object, skip over it.
          artsStream->seekg(artsHeader.AttrLength() + artsHeader.DataLength(),
                            ios::cur);
        }
        else {
          //  It's an AS matrix object.  Back up to start of header
          //  and read the whole object.
          artsStream->seekg(streamPosition,ios::beg);
          arts = new Arts();
          assert(arts != (Arts *)0);
          arts->read(*artsStream);

          if (!quiet) {
            cout << ".";
            cout.flush();
          }

          //  Add the data to the aggregator map.
          g_asmAggregatorMap.Add(*arts);

          periodAttribute = arts->FindPeriodAttribute();
          if (intervalStart == 0) {
            intervalStart = ((*periodAttribute).Period())[0];
          }

          delete(arts);

          if (hours > 0.0) {
            if ((periodAttribute->Period())[1] >
                (intervalStart + hours * 60 * 60)) {
              for (asmAggIter = g_asmAggregatorMap.begin();
                   asmAggIter != g_asmAggregatorMap.end(); asmAggIter++) {
                asMatrix = ((*asmAggIter).second)->ConvertToArtsAsMatrix();
                asMatrix->write(*out);
                delete(asMatrix);
                asMatrix = (ArtsAsMatrix *)NULL;
                delete((*asmAggIter).second);
                intervalStart = 0;
                if (!quiet) {
                  cout << "+";
                  cout.flush();
                }
              }
              g_asmAggregatorMap.erase(g_asmAggregatorMap.begin(),
                                       g_asmAggregatorMap.end());
            }
          }
        }
      }
      artsStream->close();
    }
  }

  if (g_asmAggregatorMap.size() > 0) {
    for (asmAggIter = g_asmAggregatorMap.begin();
         asmAggIter != g_asmAggregatorMap.end(); asmAggIter++) {
      asMatrix = ((*asmAggIter).second)->ConvertToArtsAsMatrix();
      asMatrix->write(*out);
      delete(asMatrix);
      asMatrix = (ArtsAsMatrix *)NULL;
      delete((*asmAggIter).second);
      intervalStart = 0;
      if (!quiet) {
        cout << "+";
        cout.flush();
      }
    }
    g_asmAggregatorMap.erase(g_asmAggregatorMap.begin(),
                             g_asmAggregatorMap.end());
    
  }

  if (!quiet)
    cout << endl;
  
  out->close();
  delete(out);

  #ifdef ARTS_DEBUG_CLASS_ALLOC
    atexit(ShowArtsObjectsNotFreed);
  #endif
  
  exit(0);
}
