//===========================================================================
//  @(#) $Name: arts++-0-9-b6 $
//  @(#) $Id: artsases.cc,v 1.14 1999/02/07 21:50:59 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed of and
//  agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb,
//  and copyrighted(C) 1998 by the University of California,
//  San Diego (UCSD), with all rights reserved.  UCSD administers the
//  CAIDA grant, NCR-9711092, under which part of this code was
//  developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

extern "C" {
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <time.h>
#include <errno.h>
}

#include <fstream.h>
#include <string>
#include <algo.h>

#include "Arts.hh"
#include "ArtsPackageVersion.hh"
#include "AsTrafficTotal.hh"

#ifdef ARTS_DEBUG_CLASS_ALLOC
  #include "ArtsDebug.hh"
#endif


extern char *tzname[2];

const string rcsid = "@(#) $Name: arts++-0-9-b6 $ $Id: artsases.cc,v 1.14 1999/02/07 21:50:59 dwm Exp $";
static const ArtsPackageVersion  k_artsVersion = ArtsPackageVersion(rcsid);

static bool      g_srcAsSpecified = false;
static bool      g_dstAsSpecified = false;
static bool      g_showRate = false;
static bool      g_showTotals = false;
static uint16_t  g_srcAs = 0;
static uint16_t  g_dstAs = 0;
static int       g_totalAs = -1;

//-------------------------------------------------------------------------
//   static bool                               
//   AsEntryMatches(vector<ArtsAsMatrixEntry>::const_iterator & asEntry) 
//.........................................................................
//  
//-------------------------------------------------------------------------
static bool
AsEntryMatches(vector<ArtsAsMatrixEntry>::const_iterator & asEntry)
{
  if (g_srcAsSpecified)
    if (g_srcAs != asEntry->Src())
      return(false);
  if (g_dstAsSpecified)
    if (g_dstAs != asEntry->Dst())
      return(false);

  return(true);
}

//----------------------------------------------------------------------------
//    static void ShowAsMatrixTotals(const Arts & arts, bool sortByPkts)   
//............................................................................
//  
//----------------------------------------------------------------------------
static void ShowAsMatrixTotals(const Arts & arts, bool sortByPkts)
{
  struct tm       *localTm;
  time_t           periodTimes[2];
  struct in_addr   inAddr;
  
  vector<ArtsAttribute>::const_iterator  periodAttribute;
  vector<ArtsAttribute>::const_iterator  hostAttribute;
  vector<ArtsAttribute>::const_iterator  ifIndexAttribute;
  
  hostAttribute = arts.FindHostAttribute();
  if (hostAttribute != arts.Attributes().end()) {
    inAddr.s_addr = hostAttribute->Host();
    cout << "router:  " << inet_ntoa(inAddr) << endl;
  }

  ifIndexAttribute = arts.FindIfIndexAttribute();
  if (ifIndexAttribute != arts.Attributes().end()) {
    cout << "ifIndex: " << ifIndexAttribute->IfIndex() << endl;
  }
    
  periodAttribute = arts.FindPeriodAttribute();
  if (periodAttribute != arts.Attributes().end()) {
    memcpy(periodTimes,periodAttribute->Period(),8);
    localTm = localtime(&(periodTimes[0]));
    cout << "period:  " << setfill('0')
         << setw(2) << (int)localTm->tm_mon+1 << "/"
         << setw(2) << localTm->tm_mday << "/"
         << setw(4) << localTm->tm_year + 1900 << " "
         << setw(2) << localTm->tm_hour << ":"
         << setw(2) << localTm->tm_min << ":"
         << setw(2) << localTm->tm_sec << " - ";
    localTm = localtime(&(periodTimes[1]));
    cout << setw(2) << (int)localTm->tm_mon+1 << "/"
         << setw(2) << localTm->tm_mday << "/"
         << setw(4) << localTm->tm_year + 1900 << " "
         << setw(2) << localTm->tm_hour << ":"
         << setw(2) << localTm->tm_min << ":"
         << setw(2) << localTm->tm_sec << " " << tzname[localTm->tm_isdst]
         << endl;
  }

  time_t timeInterval = periodTimes[1] - periodTimes[0];
  
  map<uint16_t,AsTrafficTotal,less<uint16_t> >  asTotalMap;
  vector<ArtsAsMatrixEntry>::const_iterator  asEntry;
  for (asEntry = arts.AsMatrixData()->AsEntries().begin();
       asEntry != arts.AsMatrixData()->AsEntries().end();
       asEntry++) {
    if ((*asEntry).Src() == (uint16_t)g_totalAs || g_totalAs == -1) {
      asTotalMap[asEntry->Src()].As(asEntry->Src());
      asTotalMap[asEntry->Src()].OutPkts(asTotalMap[asEntry->Src()].OutPkts() +
                                         asEntry->Pkts());
      asTotalMap[asEntry->Src()].OutBytes(asTotalMap[asEntry->Src()].OutBytes() + asEntry->Bytes());
    }
    if ((*asEntry).Dst() == (uint16_t)g_totalAs || g_totalAs == -1) {
      asTotalMap[asEntry->Dst()].As(asEntry->Dst());
      asTotalMap[asEntry->Dst()].InPkts(asTotalMap[asEntry->Dst()].InPkts() +
                                        asEntry->Pkts());
      asTotalMap[asEntry->Dst()].InBytes(asTotalMap[asEntry->Dst()].InBytes() +
                                         asEntry->Bytes());
    }
  }

  map<uint16_t,AsTrafficTotal,less<uint16_t> >::const_iterator asTotalMapIter;
  vector<AsTrafficTotal>  asTotalVector;
  AsTrafficTotal          newAsTotalVectorEntry;
  
  for (asTotalMapIter = asTotalMap.begin();
       asTotalMapIter != asTotalMap.end(); asTotalMapIter++) {
    if ((*asTotalMapIter).second.InPkts() > 0 ||
        (*asTotalMapIter).second.OutPkts() > 0) {
      newAsTotalVectorEntry.As((*asTotalMapIter).second.As());
      newAsTotalVectorEntry.InPkts((*asTotalMapIter).second.InPkts());
      newAsTotalVectorEntry.OutPkts((*asTotalMapIter).second.OutPkts());
      newAsTotalVectorEntry.InBytes((*asTotalMapIter).second.InBytes());
      newAsTotalVectorEntry.OutBytes((*asTotalMapIter).second.OutBytes());
      asTotalVector.push_back(newAsTotalVectorEntry);
    }
  }
  if (sortByPkts)
    sort (asTotalVector.begin(),asTotalVector.end(),
          AsTrafficTotalGreaterPkts());
  else
    sort (asTotalVector.begin(),asTotalVector.end(),
          AsTrafficTotalGreaterBytes());
  cout << setfill(' ');
  cout << setw(8) << "AS";
  if (g_showRate) {
    cout << setw(15) << "InPkts/sec"
         << setw(15) << "InBits/sec"
         << setw(15) << "OutPkts/sec"
         << setw(15) << "OutBits/sec"
         << endl;
  }
  else {
    cout << setw(15) << "Input Pkts"
         << setw(15) << "Input Bytes"
         << setw(15) << "Output Pkts"
         << setw(15) << "Output Bytes"
         << endl;
  }
  cout << setw(8) << "------"
       << setw(15) << "-------------"
       << setw(15) << "-------------"
       << setw(15) << "-------------"
       << setw(15) << "-------------"
       << endl;

  vector<AsTrafficTotal>::const_iterator  asTotalVectIter;
  for (asTotalVectIter = asTotalVector.begin();
       asTotalVectIter != asTotalVector.end(); asTotalVectIter++) {
    cout << setw(8) << (*asTotalVectIter).As();
    if (g_showRate) {
      cout << setw(15) << (*asTotalVectIter).InPkts() / timeInterval
           << setw(15)
           << (uint64_t)(((*asTotalVectIter).InBytes() * 8.0) / timeInterval)
           << setw(15) << (*asTotalVectIter).OutPkts() / timeInterval
           << setw(15)
           << (uint64_t)(((*asTotalVectIter).OutBytes() * 8.0) / timeInterval)
           << endl;
    }
    else {
      cout << setw(15) << (*asTotalVectIter).InPkts()
           << setw(15) << (*asTotalVectIter).InBytes()
           << setw(15) << (*asTotalVectIter).OutPkts()
           << setw(15) << (*asTotalVectIter).OutBytes()
           << endl;
    }
  }
  return;
}

//-------------------------------------------------------------------------
//      static void ShowAsMatrix(const Arts & arts, bool sortByPkts)
//.........................................................................
//  
//-------------------------------------------------------------------------
static void ShowAsMatrix(const Arts & arts, bool sortByPkts)
{
  struct tm       *localTm;
  time_t           periodTimes[2];
  struct in_addr   inAddr;
  
  vector<ArtsAttribute>::const_iterator  periodAttribute;
  vector<ArtsAttribute>::const_iterator  hostAttribute;
  vector<ArtsAttribute>::const_iterator  ifIndexAttribute;
  
  hostAttribute = arts.FindHostAttribute();
  if (hostAttribute != arts.Attributes().end()) {
    inAddr.s_addr = hostAttribute->Host();
    cout << "router:  " << inet_ntoa(inAddr) << endl;
  }

  ifIndexAttribute = arts.FindIfIndexAttribute();
  if (ifIndexAttribute != arts.Attributes().end()) {
    cout << "ifIndex: " << ifIndexAttribute->IfIndex() << endl;
  }
    
  periodAttribute = arts.FindPeriodAttribute();
  if (periodAttribute != arts.Attributes().end()) {
    memcpy(periodTimes,periodAttribute->Period(),8);
    localTm = localtime(&(periodTimes[0]));
    cout << "period:  " << setfill('0')
         << setw(2) << (int)localTm->tm_mon+1 << "/"
         << setw(2) << localTm->tm_mday << "/"
         << setw(4) << localTm->tm_year + 1900 << " "
         << setw(2) << localTm->tm_hour << ":"
         << setw(2) << localTm->tm_min << ":"
         << setw(2) << localTm->tm_sec << " - ";
    localTm = localtime(&(periodTimes[1]));
    cout << setw(2) << (int)localTm->tm_mon+1 << "/"
         << setw(2) << localTm->tm_mday << "/"
         << setw(4) << localTm->tm_year + 1900 << " "
         << setw(2) << localTm->tm_hour << ":"
         << setw(2) << localTm->tm_min << ":"
         << setw(2) << localTm->tm_sec << " " << tzname[localTm->tm_isdst]
         << endl;
  }

  time_t timeInterval = periodTimes[1] - periodTimes[0];
  
  cout << setfill(' ');
  cout << setw(8) << "Src AS"
       << setw(8) << "Dst AS"
       << setw(15) << "Pkts"
       << setw(15) << "Pkts/sec"
       << setw(15) << "Bytes"
       << setw(15) << "Bits/sec"
       << endl;
  cout << setw(8) << "------"
       << setw(8) << "------"
       << setw(15) << "-------------"
       << setw(15) << "-------------"
       << setw(15) << "-------------"
       << setw(15) << "-------------"
       << endl;

  if (sortByPkts)
    arts.AsMatrixData()->SortEntriesByPkts();
  else
    arts.AsMatrixData()->SortEntriesByBytes();

  vector<ArtsAsMatrixEntry>::const_iterator  asEntry;
  for (asEntry = arts.AsMatrixData()->AsEntries().begin();
       asEntry != arts.AsMatrixData()->AsEntries().end();
       asEntry++) {
    if (AsEntryMatches(asEntry)) {
      cout << setw(8) << asEntry->Src()
           << setw(8) << asEntry->Dst()
           << setw(15) << asEntry->Pkts()
           << setw(15) << (double)asEntry->Pkts() / timeInterval
           << setw(15) << asEntry->Bytes()
           << setw(15) << ((double)asEntry->Bytes() * 8.0) / timeInterval
           << endl;
    }
  }
  cout << endl;
  
  return;
}
  
//-------------------------------------------------------------------------
//                      void Usage(const char *argv0)                      
//.........................................................................
//  
//-------------------------------------------------------------------------
void Usage(const char *argv0)
{
  cerr << "Usage: " << argv0 << " [-v] [-p] [-t AS] [-r] [-s srcAS]"
       << " [-d dstAS] file(s)" << endl;
  return;
}

//-------------------------------------------------------------------------
//                     int main(int argc, char *argv[])                    
//.........................................................................
//  
//-------------------------------------------------------------------------
int main(int argc, char *argv[])
{
  Arts        *arts;
  ArtsHeader   artsHeader;
  ifstream    *artsStream;
  extern char *optarg;
  extern int   optind;
  int          optChar;
  bool         sortByPkts = false;
  
  if (argc < 2) {
    Usage(argv[0]);
    exit(1);
  }

  while ((optChar = getopt(argc,argv,"prs:t:d:v")) != EOF) {
    switch (optChar) {
      case 'p':
        sortByPkts = true;
        break;
      case 'r':
        g_showRate = true;
        break;
      case 's':
        g_srcAsSpecified = true;
        g_srcAs = (uint16_t)atoi(optarg);
        break;
      case 't':
        g_showTotals = true;
        g_totalAs = atoi(optarg);
        break;
      case 'd':
        g_dstAsSpecified = true;
        g_dstAs = (uint16_t)atoi(optarg);
        break;
      case 'v':
        cerr << k_artsVersion.Name() << "  " << k_artsVersion.Id() << endl;
        exit(0);
        break;
      case '?':
      default:
        Usage(argv[0]);
        exit(1);
        break;
    }
  }

  if (optind >= argc) {
    Usage(argv[0]);
    exit(1);
  }
  
  streampos  streamPosition;
  
  for (int fileNum = optind; fileNum < argc; fileNum++) {
    artsStream = new ifstream(argv[fileNum]);
    if (! artsStream || !*artsStream) {
      cerr << "[E] unable to open input file '" << argv[fileNum] << "': "
           << strerror(errno) << endl;
      continue;
    }
    
    while (*artsStream) {
      //  save our stream position, we might want to back up
      //  and read a whole ARTS object.
      streamPosition = artsStream->tellg();

      //  Read the object header from the stream.  We'll use it to
      //  determine whether or not the object is an AS matrix
      //  object.
      artsHeader.read(*artsStream);
      if (! *artsStream)
        break;
              
      if (artsHeader.Identifier() != artsC_OBJECT_AS_MATRIX) {
        //  Not an AS matrix object, skip over it.
        artsStream->seekg(artsHeader.AttrLength() + artsHeader.DataLength(),
                          ios::cur);
      }
      else {
        //  It's an AS matrix object.  Back up to start of header
        //  and read the whole object.
        artsStream->seekg(streamPosition,ios::beg);
        arts = new Arts();
        assert(arts != (Arts *)0);
        arts->read(*artsStream);
        if (arts->AsMatrixData()->AsEntries().size() > 0) {
          if (g_showTotals) {
            ShowAsMatrixTotals(*arts,sortByPkts);
          }
          else {
            ShowAsMatrix(*arts,sortByPkts);
          }
        }
        delete(arts);
      }
    } //.......... while (*artsStream)
    
    artsStream->close();
    
  } //.......... for (int fileNum = optind; ...)

  #ifdef ARTS_DEBUG_CLASS_ALLOC
    atexit(ShowArtsObjectsNotFreed);
  #endif
  
  exit(0);
}
