//===========================================================================
// @(#) $Name: arts++-0-9-b6 $
// @(#) $Id: artsdump.cc,v 1.8 1999/02/04 19:56:25 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

extern "C" {
#include <errno.h>
}

#include <string>
#include <fstream.h>

#include "Arts.hh"
#include "ArtsPackageVersion.hh"
#ifdef ARTS_DEBUG_CLASS_ALLOC
  #include "ArtsDebug.hh"
#endif

static const string rcsid = "@(#) $Name: arts++-0-9-b6 $ $Id: artsdump.cc,v 1.8 1999/02/04 19:56:25 dwm Exp $";
static const ArtsPackageVersion  k_artsVersion = ArtsPackageVersion(rcsid);

//-------------------------------------------------------------------------
//                      void Usage(const char *argv0)
//.........................................................................
//
//-------------------------------------------------------------------------
void Usage(const char *argv0)
{
  cerr << "Usage: " << argv0 << " [-v] file(s)" << endl;
  return;
}

//-------------------------------------------------------------------------
//                     int main(int argc, char *argv[])                    
//.........................................................................
//  
//-------------------------------------------------------------------------
int main(int argc, char *argv[])
{
  ifstream    *artsStream;
  Arts        *arts;
  extern char *optarg;
  extern int   optind;
  int          optChar;
  
  while ((optChar = getopt(argc,argv,"v")) != EOF) {
    switch (optChar) {
      case 'v':
        cerr << k_artsVersion.Name() << "  " << k_artsVersion.Id() << endl;
        exit(0);
        break;
      case '?':
      default:
        Usage(argv[0]);
        exit(1);
        break;
    }
  }

  if (optind >= argc) {
    Usage(argv[0]);
    exit(1);
  }
  
  for (int fileNum = optind; fileNum < argc; fileNum++) {
    artsStream = new ifstream(argv[fileNum]);
    if (!artsStream || !(*artsStream)) {
      cerr << "[E] unable to open input file '" << argv[fileNum] << "': "
           << strerror(errno) << endl;
      continue;
    }
    
    while (*artsStream) {
      arts = new Arts();
      assert(arts != (Arts *)0);
      arts->read(*artsStream);
      if (! *artsStream)
        break;
      cout << *arts << endl;
      delete(arts);
    }
    artsStream->close();
    delete(artsStream);
  }

  #ifdef ARTS_DEBUG_CLASS_ALLOC
    atexit(ShowArtsObjectsNotFreed);
  #endif

  exit(0);
}
