//===========================================================================
//  @(#) $Name: arts++-0-9-b6 $
//  @(#) $Id: artsintfms.cc,v 1.4 1999/02/04 19:56:26 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed of and
//  agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb,
//  and copyrighted(C) 1998 by the University of California,
//  San Diego (UCSD), with all rights reserved.  UCSD administers the
//  CAIDA grant, NCR-9711092, under which part of this code was
//  developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

extern "C" {
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <time.h>
#include <errno.h>
}

#include <fstream.h>
#include <string>

#include "Arts.hh"
#include "ArtsPackageVersion.hh"
#ifdef ARTS_DEBUG_CLASS_ALLOC
  #include "ArtsDebug.hh"
#endif

extern char *tzname[2];

const string rcsid = "@(#) $Name: arts++-0-9-b6 $ $Id: artsintfms.cc,v 1.4 1999/02/04 19:56:26 dwm Exp $";
static const ArtsPackageVersion  k_artsVersion = ArtsPackageVersion(rcsid);

static bool      g_srcInterfaceSpecified = false;
static bool      g_dstInterfaceSpecified = false;
static uint16_t  g_srcInterface = 0;
static uint16_t  g_dstInterface = 0;

//-------------------------------------------------------------------------
//   static bool                               
//   InterfaceEntryMatches(vector<ArtsInterfaceMatrixEntry>::const_iterator & interfaceEntry) 
//.........................................................................
//  
//-------------------------------------------------------------------------
static bool
InterfaceEntryMatches(vector<ArtsInterfaceMatrixEntry>::const_iterator & interfaceEntry)
{
  if (g_srcInterfaceSpecified)
    if (g_srcInterface != interfaceEntry->Src())
      return(false);
  if (g_dstInterfaceSpecified)
    if (g_dstInterface != interfaceEntry->Dst())
      return(false);

  return(true);
}

//-------------------------------------------------------------------------
//      static void ShowInterfaceMatrix(const Arts & arts, bool sortByPkts)
//.........................................................................
//  
//-------------------------------------------------------------------------
static void ShowInterfaceMatrix(const Arts & arts, bool sortByPkts)
{
  struct tm       *localTm;
  time_t           periodTimes[2];
  struct in_addr   inAddr;
  
  vector<ArtsAttribute>::const_iterator  periodAttribute;
  vector<ArtsAttribute>::const_iterator  hostAttribute;
  vector<ArtsAttribute>::const_iterator  ifIndexAttribute;
  
  hostAttribute = arts.FindHostAttribute();
  if (hostAttribute != arts.Attributes().end()) {
    inAddr.s_addr = hostAttribute->Host();
    cout << "router:  " << inet_ntoa(inAddr) << endl;
  }

  ifIndexAttribute = arts.FindIfIndexAttribute();
  if (ifIndexAttribute != arts.Attributes().end()) {
    cout << "ifIndex: " << ifIndexAttribute->IfIndex() << endl;
  }
    
  periodAttribute = arts.FindPeriodAttribute();
  if (periodAttribute != arts.Attributes().end()) {
    memcpy(periodTimes,periodAttribute->Period(),8);
    localTm = localtime(&(periodTimes[0]));
    cout << "period:  " << setfill('0')
         << setw(2) << (int)localTm->tm_mon+1 << "/"
         << setw(2) << localTm->tm_mday << "/"
         << setw(4) << localTm->tm_year + 1900 << " "
         << setw(2) << localTm->tm_hour << ":"
         << setw(2) << localTm->tm_min << ":"
         << setw(2) << localTm->tm_sec << " - ";
    localTm = localtime(&(periodTimes[1]));
    cout << setw(2) << (int)localTm->tm_mon+1 << "/"
         << setw(2) << localTm->tm_mday << "/"
         << setw(4) << localTm->tm_year + 1900 << " "
         << setw(2) << localTm->tm_hour << ":"
         << setw(2) << localTm->tm_min << ":"
         << setw(2) << localTm->tm_sec << " " << tzname[localTm->tm_isdst]
         << endl;
  }

  time_t timeInterval = periodTimes[1] - periodTimes[0];
  
  cout << setfill(' ');
  cout << setw(9) << "SrcIntf"
       << setw(9) << "DstIntf"
       << setw(15) << "Pkts"
       << setw(15) << "Pkts/sec"
       << setw(15) << "Bytes"
       << setw(15) << "Bits/sec"
       << endl;
  cout << setw(9) << "-------"
       << setw(9) << "-------"
       << setw(15) << "-------------"
       << setw(15) << "-------------"
       << setw(15) << "-------------"
       << setw(15) << "-------------"
       << endl;

  if (sortByPkts)
    arts.InterfaceMatrixData()->SortEntriesByPkts();
  else
    arts.InterfaceMatrixData()->SortEntriesByBytes();

  vector<ArtsInterfaceMatrixEntry>::const_iterator  interfaceEntry;
  for (interfaceEntry = arts.InterfaceMatrixData()->InterfaceEntries().begin();
       interfaceEntry != arts.InterfaceMatrixData()->InterfaceEntries().end();
       interfaceEntry++) {
    if (InterfaceEntryMatches(interfaceEntry)) {
      cout << setw(9) << interfaceEntry->Src()
           << setw(9) << interfaceEntry->Dst()
           << setw(15) << interfaceEntry->Pkts()
           << setw(15) << (double)interfaceEntry->Pkts() / timeInterval
           << setw(15) << interfaceEntry->Bytes()
           << setw(15) << ((double)interfaceEntry->Bytes() * 8.0) / timeInterval
           << endl;
    }
  }
  cout << endl;
  
  return;
}
  
//-------------------------------------------------------------------------
//                      void Usage(const char *argv0)                      
//.........................................................................
//  
//-------------------------------------------------------------------------
void Usage(const char *argv0)
{
  cerr << "Usage: " << argv0 << " [-v] [-p] [-s srcIntf] [-d dstIntf] file(s)"
       << endl;
  return;
}

//-------------------------------------------------------------------------
//                     int main(int argc, char *argv[])                    
//.........................................................................
//  
//-------------------------------------------------------------------------
int main(int argc, char *argv[])
{
  Arts        *arts;
  ArtsHeader   artsHeader;
  ifstream    *artsStream;
  extern char *optarg;
  extern int   optind;
  int          optChar;
  bool         sortByPkts = false;
  
  if (argc < 2) {
    Usage(argv[0]);
    exit(1);
  }

  while ((optChar = getopt(argc,argv,"ps:d:v")) != EOF) {
    switch (optChar) {
      case 'p':
        sortByPkts = true;
        break;
      case 's':
        g_srcInterfaceSpecified = true;
        g_srcInterface = (uint16_t)atoi(optarg);
        break;
      case 'd':
        g_dstInterfaceSpecified = true;
        g_dstInterface = (uint16_t)atoi(optarg);
        break;
      case 'v':
        cerr << k_artsVersion.Name() << "  " << k_artsVersion.Id() << endl;
        exit(0);
        break;
      case '?':
      default:
        Usage(argv[0]);
        exit(1);
        break;
    }
  }

  if (optind >= argc) {
    Usage(argv[0]);
    exit(1);
  }
  
  streampos  streamPosition;
  
  for (int fileNum = optind; fileNum < argc; fileNum++) {
    artsStream = new ifstream(argv[fileNum]);
    if (! artsStream || !*artsStream) {
      cerr << "[E] unable to open input file '" << argv[fileNum] << "': "
           << strerror(errno) << endl;
      continue;
    }
    
    while (*artsStream) {
      //  save our stream position, we might want to back up
      //  and read a whole ARTS object.
      streamPosition = artsStream->tellg();

      //  Read the object header from the stream.  We'll use it to
      //  determine whether or not the object is an interface matrix
      //  object.
      artsHeader.read(*artsStream);
      if (! *artsStream)
        break;
              
      if (artsHeader.Identifier() != artsC_OBJECT_INTERFACE_MATRIX) {
        //  Not an interface matrix object, skip over it.
        artsStream->seekg(artsHeader.AttrLength() + artsHeader.DataLength(),
                          ios::cur);
      }
      else {
        //  It's an interface matrix object.  Back up to start of header
        //  and read the whole object.
        artsStream->seekg(streamPosition,ios::beg);
        arts = new Arts();
        assert(arts != (Arts *)0);
        arts->read(*artsStream);
        if (arts->InterfaceMatrixData()->InterfaceEntries().size() > 0) {
          ShowInterfaceMatrix(*arts,sortByPkts);
        }
        delete(arts);
      }
    } //.......... while (*artsStream)
    
    artsStream->close();
    
  } //.......... for (int fileNum = optind; ...)

  #ifdef ARTS_DEBUG_CLASS_ALLOC
    atexit(ShowArtsObjectsNotFreed);
  #endif
  
  exit(0);
}
