//===========================================================================
//  @(#) $Name: arts++-0-9-b6 $
//  @(#) $Id: jcprotobar.cc,v 1.1 1998/09/06 09:00:48 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed of and
//  agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb,
//  and copyrighted(C) 1998 by the University of California,
//  San Diego (UCSD), with all rights reserved.  UCSD administers the
//  CAIDA grant, NCR-9711092, under which part of this code was
//  developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

extern "C" {
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <time.h>
#include <netdb.h>
}

#include <fstream.h>
#include <string>

#include "ArtsProtocolTableAggregator.hh"
#include "Arts.hh"

extern int errno;

const string rcsid = "@(#) $Name: arts++-0-9-b6 $ $Id: jcprotobar.cc,v 1.1 1998/09/06 09:00:48 dwm Exp $";

static char *g_rtrName = NULL;

//-------------------------------------------------------------------------
//                      void Usage(const char *argv0)                      
//.........................................................................
//  
//-------------------------------------------------------------------------
void Usage(const char *argv0)
{
  cerr << "Usage: " << argv0
       << " -c codeBase infile(s)"
       << endl;
  return;
}

//-------------------------------------------------------------------------
//            static ArtsProtocolTableAggregator *                     
//            GenerateProtocolTableAggregator(int argc, char *argv[])  
//.........................................................................
//  
//-------------------------------------------------------------------------
static ArtsProtocolTableAggregator *
GenerateProtocolTableAggregator(int argc, char *argv[])
{
  Arts        *arts;
  ArtsHeader   artsHeader;
  ifstream    *artsStream;
  ofstream    *out;

  ArtsProtocolTableAggregator  *protoAgg = (ArtsProtocolTableAggregator *)NULL;

  streampos  streamPosition;
  
  for (int fileNum = 2; fileNum < argc; fileNum++) {
    artsStream = new ifstream(argv[fileNum]);
    if (artsStream) {
      while (*artsStream) {
        //  save our stream position, we might want to back up
        //  and read a whole ARTS object.
        streamPosition = artsStream->tellg();

        //  Read the object header from the stream.  We'll use it to
        //  determine whether or not the object is an AS matrix
        //  object.
        artsHeader.read(*artsStream);
        if (! *artsStream)
          break;
        
        if (artsHeader.Identifier() != artsC_OBJECT_PROTO) {
          //  Not a protocol table object, skip over it.
          artsStream->seekg(artsHeader.AttrLength() + artsHeader.DataLength(),
                            ios::cur);
        }
        else {
          //  It's a protocol table object.  Back up to start of header
          //  and read the whole object.
          artsStream->seekg(streamPosition,ios::beg);
          arts = new Arts();
          arts->read(*artsStream);

          if (!g_rtrName) {
            vector<ArtsAttribute>::const_iterator   hostAttribute;
            struct in_addr                          inAddr;
            struct hostent                         *hostEntry;
            
            hostAttribute = arts->FindHostAttribute();
            if (hostAttribute) {
              inAddr.s_addr = hostAttribute->Host();
              hostEntry = gethostbyaddr((char *)&inAddr.s_addr,
                                        4,AF_INET);
              if (hostEntry) {
                g_rtrName = (char *)strdup(hostEntry->h_name);
              }
              else {
                g_rtrName = (char *)strdup(inet_ntoa(inAddr));
              }
            }
          }
          
          // cout << ".";
          // cout.flush();
          
          if (protoAgg) {
            protoAgg->Add(*arts);
          }
          else {
            protoAgg = new ArtsProtocolTableAggregator(*arts);
          }
          delete(arts);
        }
      }
      artsStream->close();
    }
  }
  return(protoAgg);
}

static map<uint8_t,vector<uint64_t>,less<uint8_t> > *
GenerateTimeSeriesData(vector<ArtsProtocolTableEntry> & protoEntries,
                       vector<time_t> & timestamps,
                       int argc, char *argv[])
{
  vector<uint64_t>  newVector;
  map<uint8_t,vector<uint64_t>,less<uint8_t> > *timeSeriesMap =
    new map<uint8_t,vector<uint64_t>,less<uint8_t> >;

  vector<ArtsProtocolTableEntry>::iterator protoIter;
  for (protoIter = protoEntries.begin(); protoIter != protoEntries.end();
       protoIter++) {
    (*timeSeriesMap)[(*protoIter).ProtocolNumber()] = newVector;
  }

  uint32_t     numTimeslots = 0;
  Arts        *arts;
  ArtsHeader   artsHeader;
  ifstream    *artsStream;
  streampos    streamPosition;
  vector<ArtsAttribute>::const_iterator  periodAttribute;
  
  map<uint8_t,vector<uint64_t>,less<uint8_t> >::iterator  timeSeriesMapIter;
  
  for (int fileNum = 2; fileNum < argc; fileNum++) {
    artsStream = new ifstream(argv[fileNum]);
    if (artsStream) {
      while (*artsStream) {
        //  save our stream position, we might want to back up
        //  and read a whole ARTS object.
        streamPosition = artsStream->tellg();

        //  Read the object header from the stream.  We'll use it to
        //  determine whether or not the object is a protocol table
        //  object.
        artsHeader.read(*artsStream);
        if (! *artsStream)
          break;
        
        if (artsHeader.Identifier() != artsC_OBJECT_PROTO) {
          //  Not a protocol table object, skip over it.
          artsStream->seekg(artsHeader.AttrLength() + artsHeader.DataLength(),
                            ios::cur);
        }
        else {
          //  It's a protocol table object.  Back up to start of header
          //  and read the whole object.
          artsStream->seekg(streamPosition,ios::beg);
          arts = new Arts();
          arts->read(*artsStream);

          for (timeSeriesMapIter = (*timeSeriesMap).begin();
               timeSeriesMapIter != (*timeSeriesMap).end();
               timeSeriesMapIter++) {
            (*timeSeriesMapIter).second.push_back(0);
          }
          
          periodAttribute = arts->FindPeriodAttribute();
          timestamps.push_back(((*periodAttribute).Period())[0]);
          
          vector<ArtsProtocolTableEntry>::iterator protoEntry;
          for (protoEntry = arts->ProtocolTableData()->ProtocolEntries().begin();
               protoEntry != arts->ProtocolTableData()->ProtocolEntries().end();
               protoEntry++) {
            timeSeriesMapIter =
              (*timeSeriesMap).find((*protoEntry).ProtocolNumber());
            if (timeSeriesMapIter != (*timeSeriesMap).end()) {
              (*timeSeriesMapIter).second[numTimeslots] +=
                (uint64_t)(((*protoEntry).Bytes() * (double)8.0) /
                           (((*periodAttribute).Period())[1] -
                            ((*periodAttribute).Period())[0]));
            }
          }
          delete(arts);
          numTimeslots++;
        }
      }
      artsStream->close();
    }
  }
  return(timeSeriesMap);
}

//-------------------------------------------------------------------------
// static void                               
// GenerateHtml(map<uint8_t,vector<uint64_t>,less<uint8_t> > *protoCounterMap, 
//              vector<ArtsProtocolTableEntry> & protocolEntries, 
//              vector<time_t> & timestamps, 
//              const char *codeBase)        
//.........................................................................
//  
//-------------------------------------------------------------------------
static void
GenerateHtml(map<uint8_t,vector<uint64_t>,less<uint8_t> > *protoCounterMap,
             vector<ArtsProtocolTableEntry> & protocolEntries,
             vector<time_t> & timestamps,
             const char *codeBase)
{
  struct tm  *gmtTm;
  time_t  timeBase = *(timestamps.begin());
  
  cout << "<html>\n<body>\n"
       << "<applet CODEBASE=\"" << codeBase << "\" "
       << "CODE=\"jclass/chart/JCChartApplet.class\" "
       << "WIDTH=100%>" << endl
       << "<PARAM name=\"chart.header.text\" "
       << "value=\"" << g_rtrName
       << ": traffic by IP protocol\">" << endl
       << "<PARAM name=\"chart.header.font\" value=\"TimesRoman-bold-20\">"
       << endl
       << "<PARAM name=\"chart.header.isShowing\" value=\"true\">" << endl
       << "<PARAM name=\"chart.data.chartType\" value=\"STACKING_BAR\">"
       << endl
       << "<PARAM name=\"data.Bar.clusterWidth\" value=\"100\">" << endl
       << "<PARAM name=\"chart.legend.isShowing\" value=\"true\">" << endl
       << "<PARAM name=\"chart.rotateTrigger\" value=\"CTRL\">" << endl
       << "<PARAM name=\"chart.chartArea.rotation\" value=\"0\">" << endl
       << "<PARAM name=\"chart.chartArea.depth\" value=\"2\">" << endl
       << "<PARAM name=\"chart.chartArea.elevation\" value=\"0\">" << endl
       << "<PARAM name=\"chart.xaxis.annotationMethod\" value=\"TIME_LABELS\">"
       << endl;
  gmtTm = gmtime(&timeBase);
  cout << "<PARAM name=\"chart.xaxis.timeBase\" value=\""
       << gmtTm->tm_mon + 1 << "/" << gmtTm->tm_mday << "/"
       << gmtTm->tm_year << " " << setfill('0')
       << setw(2) << gmtTm->tm_hour << ":" << setw(2) << gmtTm->tm_min
       << "\">" << endl << setfill(' ')
       << "<PARAM name=\"chart.xaxis.timeFormat\" value=\"%m/%d %H:%M\">"
       << endl
       << "<PARAM name=\"chart.xaxis.title.text\" "
       << "value=\"Date and Time (GMT)\">" << endl
       << "<PARAM name=\"chart.yaxis.title.text\" value=\"bits/sec\">" << endl
       << "<PARAM name=\"chart.yaxis.title.rotation\" value=\"90\">" << endl
       << "<PARAM name=\"chart.yaxis.title.placement\" value=\"west\">" << endl
       << "<PARAM name=\"chart.background\" value=\"#D1E1FF\">" << endl
       << "<PARAM name=\"chart.chartarea.plotArea.background\""
       << " value=\"#BCC7E6\">" << endl
       << "<PARAM name=\"chart.ZoomTrigger\" value=\"NONE\">" << endl;

  cout << "<PARAM name=\"data\" value=\"ARRAY 'Protocol Number' "
       << protocolEntries.size() << " " << timestamps.size() << " "
       << "HOLE -1 ";

  vector<time_t>::iterator tsIter;

  for (tsIter = timestamps.begin(); tsIter != timestamps.end(); tsIter++) {
    cout << " " << *tsIter - *(timestamps.begin());
  }
  cout << " ";
  
  map<uint8_t,vector<uint64_t>,less<uint8_t> >::iterator  tsMapIter;
  vector<uint64_t>::iterator                              tsMapValueIter;
  vector<ArtsProtocolTableEntry>::iterator                protoIter;

  for (protoIter = protocolEntries.begin();
       protoIter != protocolEntries.end(); protoIter++) {
    tsMapIter = (*protoCounterMap).find((*protoIter).ProtocolNumber());
    cout << "'" << (int)(*tsMapIter).first << "' ";
    for (tsMapValueIter = (*tsMapIter).second.begin();
         tsMapValueIter != (*tsMapIter).second.end();
         tsMapValueIter++) {
      cout << (*tsMapValueIter) << " ";
    }
  }
  cout << "\">" << endl;
  
  cout << "</applet>" << endl
       << "</body>" << endl
       << "</html>" << endl;
  return;
}

//-------------------------------------------------------------------------
//                     int main(int argc, char *argv[])                    
//.........................................................................
//  
//-------------------------------------------------------------------------
int main(int argc, char *argv[])
{
  Arts        *arts;
  ArtsHeader   artsHeader;
  ifstream    *artsStream;
  char        *codeBase = NULL;
  char        *dataFilename = NULL;
  uint32_t     numTimeslots = 0;
  extern int   optind;
  extern char *optarg;
  int          optChar;
  
  if (argc < 4) {
    Usage(argv[0]);
    exit(1);
  }

  while ((optChar = getopt(argc,argv,"c:w:d:n:")) != EOF) {
    switch (optChar) {
      case 'c':
        codeBase = (char *)strdup(optarg);
        break;
      default:
        break;
    }
  }

  if (!codeBase) {
    Usage(argv[0]);
  }
  
  ArtsProtocolTableAggregator  *protoAgg =
    GenerateProtocolTableAggregator(argc,argv);

  ArtsProtocolTable *protoTable = protoAgg->ConvertToArtsProtocolTable();
  
  //  O.K., I now have a list of the top destination ASes by bytes of traffic.
  //  I now need to go through the input data a second time, to generate
  //  time-series data for each destination AS in my destination AS list.
  //  I also need to generate the timestamp information for the plot.
  //
  //  I think the way to do this is to build a map keyed by dst AS number,
  //  which we'll reuse with each ARTS object we read from input files.
  //  The values in the map should contain vectors of counters, with
  //  each counter holding pkts and bytes from the destination AS during a
  //  given time interval?  Then at the end we can spit this data out
  //  in a form readable by jclass chart.

  vector<time_t>  timestamps;
  
  map<uint8_t,vector<uint64_t>,less<uint8_t> >  *protoCounterMap =
    GenerateTimeSeriesData(protoTable->ProtocolEntries(),timestamps,argc,argv);

  protoTable->ProtocolTableData()->SortEntriesByBytes();
  
  GenerateHtml(protoCounterMap,protoTable->ProtocolEntries(),
               timestamps,codeBase);
  
  cout.flush();
  
  delete(protoCounterMap);
  delete(protoTable);

  delete(protoAgg);
}
