//===========================================================================
//  $Name: arts++-0-9-b6 $
//  $Id: ArtsBgp4AsPathAttribute.hh,v 1.2 1998/08/14 03:31:15 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSBGP4ASPATHATTRIBUTE_HH_
#define _ARTSBGP4ASPATHATTRIBUTE_HH_

extern "C" {
#include "caida_t.h"
}

#include "ArtsBgp4AsPathSegment.hh"

//---------------------------------------------------------------------------
//  class ArtsBgp4AsPathAttribute
//---------------------------------------------------------------------------
//  This class abstracts a BGP4 AS path attribute.  It contains a
//  vector of ArtsBgp4AsPathSegment objects.
//---------------------------------------------------------------------------
class ArtsBgp4AsPathAttribute
{
public:
  //-------------------------------------------------------------------------
  //                      uint8_t  NumSegments() const
  //.........................................................................
  //  Returns the number of ArtsBgp4AsPathSegment objects contained
  //  in the ArtsBgp4AsPathAttribute.
  //-------------------------------------------------------------------------
  uint8_t  NumSegments() const;
  
  //-------------------------------------------------------------------------
  //                uint8_t  NumSegments(uint8_t numSegments)
  //.........................................................................
  //  Sets and returns the number of ArtsBgp4AsPathSegment objects
  //  contained in the ArtsBgp4AsPathAttribute.
  //-------------------------------------------------------------------------
  uint8_t  NumSegments(uint8_t numSegments);
  
  //-------------------------------------------------------------------------
  //            vector<ArtsBgp4AsPathSegment> & Segments() const
  //.........................................................................
  //  Returns a reference to the vector of ArtsBgp4AsPathSegment objects
  //  in the ArtsBgp4AsPathAttribute.  Note that we can call this member
  //  on a const object even though we don't return a const reference;
  //  the vector of ArtsBgp4AsPathSegment objects is mutable.
  //-------------------------------------------------------------------------
  vector<ArtsBgp4AsPathSegment> & Segments() const;

  //-------------------------------------------------------------------------
  //           operator = (const ArtsBgp4AsPathAttribute & asPath)
  //.........................................................................
  //  Overloaded '=' operator for assigning the value of one
  //  ArtsBgp4AsPathAttribute to another.
  //-------------------------------------------------------------------------
  ArtsBgp4AsPathAttribute &
  operator = (const ArtsBgp4AsPathAttribute & asPath);
  
private:
  mutable vector<ArtsBgp4AsPathSegment>   _segments;
};

#endif  // _ARTSBGP4ASPATHATTRIBUTE_HH_
