//===========================================================================
// @(#) $Name: arts++-0-9-b6 $
// @(#) $Id: Arts.cc,v 1.14 1999/02/04 05:28:08 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

extern "C" {
#include <stdio.h>
}

#include <string>
#include <map>

#include "Arts.hh"

static const string rcsid = "@(#) $Name: arts++-0-9-b6 $ $Id: Arts.cc,v 1.14 1999/02/04 05:28:08 dwm Exp $";

//------------------------------------------------------------------------
//                            Arts::Arts()
//........................................................................
//  constructor
//------------------------------------------------------------------------
Arts::Arts()
{
  this->_data._ipPath = (ArtsIpPathData *)0;
  this->_data._asMatrix = (ArtsAsMatrixData *)0;
  this->_data._netMatrix = (ArtsNetMatrixData *)0;
  this->_data._portTable = (ArtsPortTableData *)0;
  this->_data._portMatrix = (ArtsPortMatrixData *)0;
  this->_data._protocolTable = (ArtsProtocolTableData *)0;
  this->_data._selectedPortTable = (ArtsSelectedPortTableData *)0;
  this->_data._interfaceMatrix = (ArtsInterfaceMatrixData *)0;
  this->_data._nextHopTable = (ArtsNextHopTableData *)0;
  
  #ifdef ARTS_DEBUG_CLASS_ALLOC
    ++_numObjects;
  #endif
}

//-------------------------------------------------------------------------
//                              Arts::~Arts()                              
//.........................................................................
//  destructor
//-------------------------------------------------------------------------
Arts::~Arts()
{
  #ifdef ARTS_DEBUG_CLASS_ALLOC
    --_numObjects;
  #endif

  if (this->Attributes().size() > 0) {
    //  delete the attributes.
    this->Attributes().erase(this->Attributes().begin(),
                             this->Attributes().end());
  }
  
  switch (this->Header().Identifier()) {

    case artsC_OBJECT_IP_PATH:
      if (this->_data._ipPath != (ArtsIpPathData *)0) {
        delete(this->_data._ipPath);
        this->_data._ipPath = (ArtsIpPathData *)0;
      }
      break;

    case artsC_OBJECT_AS_MATRIX:
      if (this->_data._asMatrix != (ArtsAsMatrixData *)0) {
        delete(this->_data._asMatrix);
        this->_data._asMatrix = (ArtsAsMatrixData *)0;
      }
      break;

    case artsC_OBJECT_NET:
      if (this->_data._netMatrix != (ArtsNetMatrixData *)0) {
        delete(this->_data._netMatrix);
        this->_data._netMatrix = (ArtsNetMatrixData *)0;
      }
      break;

    case artsC_OBJECT_PORT:
      if (this->_data._portTable != (ArtsPortTableData *)0) {
        delete(this->_data._portTable);
        this->_data._portTable = (ArtsPortTableData *)0;
      }
      break;

    case artsC_OBJECT_SELECTED_PORT:
      if (this->_data._selectedPortTable !=
          (ArtsSelectedPortTableData *)0) {
        delete(this->_data._selectedPortTable);
        this->_data._selectedPortTable = (ArtsSelectedPortTableData *)0;
      }
      break;
      
    case artsC_OBJECT_PORT_MATRIX:
      if (this->_data._portMatrix != (ArtsPortMatrixData *)0) {
        delete(this->_data._portMatrix);
        this->_data._portMatrix = (ArtsPortMatrixData *)0;
      }
      break;
      
    case artsC_OBJECT_PROTO:
      if (this->_data._protocolTable != (ArtsProtocolTableData *)0) {
        delete(this->_data._protocolTable);
        this->_data._protocolTable = (ArtsProtocolTableData *)0;
      }
      break;

    case artsC_OBJECT_INTERFACE_MATRIX:
      if (this->_data._interfaceMatrix != (ArtsInterfaceMatrixData *)0) {
        delete(this->_data._interfaceMatrix);
        this->_data._interfaceMatrix = (ArtsInterfaceMatrixData *)0;
      }
      break;
      
    case artsC_OBJECT_NEXT_HOP:
      if (this->_data._nextHopTable != (ArtsNextHopTableData *)0) {
        delete(this->_data._nextHopTable);
        this->_data._nextHopTable = (ArtsNextHopTableData *)0;
      }
      break;

    default:
      break;
  }
  memset(&this->_data,0,sizeof(this->_data));
}

  
//-------------------------------------------------------------------------
//   vector<ArtsAttribute>::iterator Arts::FindCreationAttribute() const   
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
vector<ArtsAttribute>::iterator Arts::FindCreationAttribute() const
{
  vector<ArtsAttribute>::iterator  attrIter;

  for (attrIter = this->_attributes.begin();
       attrIter != this->_attributes.end(); attrIter++) {
    if ((*attrIter).Identifier() == artsC_ATTR_CREATION) {
      return(attrIter);
    }
  }
  return(this->Attributes().end());
}

//-------------------------------------------------------------------------
// vector<ArtsAttribute>::const_iterator Arts::FindPeriodAttribute() const
//.........................................................................
//  
//-------------------------------------------------------------------------
vector<ArtsAttribute>::const_iterator Arts::FindPeriodAttribute() const
{
  vector<ArtsAttribute>::const_iterator  attrIter;
  for (attrIter = this->_attributes.begin();
       attrIter != this->_attributes.end(); attrIter++) {
    if (attrIter->Identifier() == artsC_ATTR_PERIOD) {
      return(attrIter);
    }
  }
  return(this->_attributes.end());
}

//-------------------------------------------------------------------------
//  vector<ArtsAttribute>::const_iterator Arts::FindHostAttribute() const  
//.........................................................................
//  
//-------------------------------------------------------------------------
vector<ArtsAttribute>::const_iterator Arts::FindHostAttribute() const
{
  vector<ArtsAttribute>::const_iterator  attrIter;
  for (attrIter = this->_attributes.begin();
       attrIter != this->_attributes.end(); attrIter++) {
    if (attrIter->Identifier() == artsC_ATTR_HOST) {
      return(attrIter);
    }
  }
  return(this->_attributes.end());
}

//-------------------------------------------------------------------------
// vector<ArtsAttribute>::const_iterator Arts::FindIfIndexAttribute() const 
//.........................................................................
//  
//-------------------------------------------------------------------------
vector<ArtsAttribute>::const_iterator Arts::FindIfIndexAttribute() const
{
  vector<ArtsAttribute>::const_iterator  attrIter;

  for (attrIter = this->_attributes.begin();
       attrIter != this->_attributes.end(); attrIter++) {
    if (attrIter->Identifier() == artsC_ATTR_IFINDEX) {
      return(attrIter);
    }
  }
  return(this->_attributes.end());
}

//------------------------------------------------------------------------
//            void Arts::AddHostAttribute(ipv4addr_t hostAddr)
//........................................................................
//
//-------------------------------------------------------------------------
void Arts::AddHostAttribute(ipv4addr_t hostAddr)
{
  ArtsAttribute  attribute;

  attribute.Identifier(artsC_ATTR_HOST);
  attribute.Host(hostAddr);
  attribute.Format(artsC_IPV4_ADDRESS);
  attribute.Length(sizeof(uint32_t) + sizeof(uint32_t) + sizeof(ipv4addr_t));
  this->Attributes().push_back(attribute);
  this->Header().NumAttributes(this->Header().NumAttributes()+1);
  this->Header().AttrLength(this->Header().AttrLength() + attribute.Length());
  return;
}

//------------------------------------------------------------------------
//          void AddCreationAttribute(uint32_t creationTime)
//........................................................................
//
//------------------------------------------------------------------------
void Arts::AddCreationAttribute(uint32_t creationTime)
{
  ArtsAttribute  attribute;

  attribute.Identifier(artsC_ATTR_CREATION);
  attribute.Creation(creationTime);
  attribute.Format(artsC_UNIX_DATE);
  attribute.Length(sizeof(uint32_t) + sizeof(uint32_t) + sizeof(uint32_t));
  this->Attributes().push_back(attribute);
  this->Header().NumAttributes(this->Header().NumAttributes()+1);
  this->Header().AttrLength(this->Header().AttrLength() + attribute.Length());
  return;
}

//-------------------------------------------------------------------------
//   void Arts::AddPeriodAttribute(uint32_t startTime, uint32_t endTime)
//.........................................................................
//  
//-------------------------------------------------------------------------
void Arts::AddPeriodAttribute(uint32_t startTime, uint32_t endTime)
{
  ArtsAttribute  attribute;

  attribute.Identifier(artsC_ATTR_PERIOD);
  attribute.Period(startTime,endTime);
  attribute.Length(sizeof(uint32_t) + sizeof(uint32_t) +
                   sizeof(uint32_t) + sizeof(uint32_t));
  this->Attributes().push_back(attribute);
  this->Header().NumAttributes(this->Header().NumAttributes()+1);
  this->Header().AttrLength(this->Header().AttrLength() + attribute.Length());
  return;
}

//-------------------------------------------------------------------------
//             void Arts::AddIfIndexAttribute(uint16_t ifIndex)            
//.........................................................................
//  
//-------------------------------------------------------------------------
void Arts::AddIfIndexAttribute(uint16_t ifIndex)
{
  ArtsAttribute  attribute;

  attribute.Identifier(artsC_ATTR_IFINDEX);
  attribute.IfIndex(ifIndex);
  attribute.Length(sizeof(uint32_t) + sizeof(uint32_t) + sizeof(ifIndex));
  this->Attributes().push_back(attribute);
  this->Header().NumAttributes(this->Header().NumAttributes()+1);
  this->Header().AttrLength(this->Header().AttrLength() + attribute.Length());
  return;
}
  
//-------------------------------------------------------------------------
//               Arts & Arts::operator = (const Arts & arts)               
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
Arts & Arts::operator = (const Arts & arts)
{
  this->_header = arts.Header();
  this->_attributes = arts.Attributes();

  switch (this->Header().Identifier()) {
    case artsC_OBJECT_IP_PATH:
      if (this->_data._ipPath) {
        delete(this->_data._ipPath);
      }
      this->_data._ipPath = new ArtsIpPathData;
      assert(this->_data._ipPath != (ArtsIpPathData *)0);
      *(this->_data._ipPath) = *(arts.IpPathData());
      break;
      
    case artsC_OBJECT_AS_MATRIX:
      if (this->_data._asMatrix) {
        delete(this->_data._asMatrix);
      }
      this->_data._asMatrix = new ArtsAsMatrixData;
      assert(this->_data._asMatrix != (ArtsAsMatrixData *)0);
      *(this->_data._asMatrix) = *(arts.AsMatrixData());
      break;

    case artsC_OBJECT_NET:
      if (this->_data._netMatrix) {
        delete(this->_data._netMatrix);
      }
      this->_data._netMatrix = new ArtsNetMatrixData;
      assert(this->_data._netMatrix != (ArtsNetMatrixData *)0);
      *(this->_data._netMatrix) = *(arts.NetMatrixData());
      break;

    case artsC_OBJECT_PORT:
      if (this->_data._portTable) {
        delete(this->_data._portTable);
      }
      this->_data._portTable = new ArtsPortTableData;
      assert(this->_data._portTable != (ArtsPortTableData *)0);
      *(this->_data._portTable) = *(arts.PortTableData());
      break;

    case artsC_OBJECT_SELECTED_PORT:
      if (this->_data._selectedPortTable) {
        delete(this->_data._selectedPortTable);
      }
      this->_data._selectedPortTable = new ArtsSelectedPortTableData;
      assert(this->_data._selectedPortTable != (ArtsSelectedPortTableData *)0);
      *(this->_data._selectedPortTable) = *(arts.SelectedPortTableData());
      break;
      
    case artsC_OBJECT_PORT_MATRIX:
      if (this->_data._portMatrix) {
        delete(this->_data._portMatrix);
      }
      this->_data._portMatrix = new ArtsPortMatrixData;
      assert(this->_data._portMatrix != (ArtsPortMatrixData *)0);
      *(this->_data._portMatrix) = *(arts.PortMatrixData());
      break;

    case artsC_OBJECT_PROTO:
      if (this->_data._protocolTable) {
        delete(this->_data._protocolTable);
      }
      this->_data._protocolTable = new ArtsProtocolTableData;
      assert(this->_data._protocolTable != (ArtsProtocolTableData *)0);
      *(this->_data._protocolTable) = *(arts.ProtocolTableData());
      break;

    case artsC_OBJECT_INTERFACE_MATRIX:
      if (this->_data._interfaceMatrix) {
        delete(this->_data._interfaceMatrix);
      }
      this->_data._interfaceMatrix = new ArtsInterfaceMatrixData;
      assert(this->_data._interfaceMatrix != (ArtsInterfaceMatrixData *)0);
      *(this->_data._interfaceMatrix) = *(arts.InterfaceMatrixData());
      break;
      
    case artsC_OBJECT_NEXT_HOP:
      if (this->_data._nextHopTable) {
        delete(this->_data._nextHopTable);
      }
      this->_data._nextHopTable = new ArtsNextHopTableData;
      assert(this->_data._nextHopTable != (ArtsNextHopTableData *)0);
      *(this->_data._nextHopTable) = *(arts.NextHopTableData());
      break;
      
    default:
      break;
  }
  return(*this);
}

//-------------------------------------------------------------------------
//         ostream & operator << (ostream& os, const Arts & arts)      
//.........................................................................
//  
//-------------------------------------------------------------------------
ostream & operator << (ostream& os, const Arts & arts)
{
  os << arts.Header();

  for (int attrNum = 0; attrNum < arts.Header().NumAttributes();
       attrNum++) {
    os << arts.Attributes()[attrNum];
  }

  switch (arts.Header().Identifier()) {
    case artsC_OBJECT_IP_PATH:
      os << *(arts.IpPathData());
      break;
      
    case artsC_OBJECT_AS_MATRIX:
      os << *(arts.AsMatrixData());
      break;

    case artsC_OBJECT_NET:
      os << *(arts.NetMatrixData());
      break;

    case artsC_OBJECT_PORT:
      os << *(arts.PortTableData());
      break;

    case artsC_OBJECT_SELECTED_PORT:
      os << *(arts.SelectedPortTableData());
      break;
      
    case artsC_OBJECT_PORT_MATRIX:
      os << *(arts.PortMatrixData());
      break;
      
    case artsC_OBJECT_PROTO:
      os << *(arts.ProtocolTableData());
      break;

    case artsC_OBJECT_INTERFACE_MATRIX:
      os << *(arts.InterfaceMatrixData());
      break;

    case artsC_OBJECT_NEXT_HOP:
      os << *(arts.NextHopTableData());
      break;

    default:
      break;
  }
  return(os);
}

//-------------------------------------------------------------------------
//                     istream& Arts::read(istream& is)                    
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
istream& Arts::read(istream& is)
{
  uint16_t       attrNum;
  ArtsAttribute  attribute;
  
  if (is.eof())
    return(is);
  
  this->Header().read(is);
  if (is.eof())
    return(is);
  
  if (this->Header().Magic() != artsC_MAGIC)
    return(is);
      
  if (this->Header().NumAttributes() > 0) {
    this->Attributes().reserve(this->Header().NumAttributes());
    for (attrNum = 0; attrNum < this->Header().NumAttributes(); attrNum++) {
      attribute.read(is);
      if (is.eof())
        return(is);
      this->_attributes.push_back(attribute);
    }
  }

  switch (this->Header().Identifier()) {

    case artsC_OBJECT_IP_PATH:
      if (this->_data._ipPath) {
        delete(this->_data._ipPath);
      }
      this->_data._ipPath = new ArtsIpPathData;
      assert(this->_data._ipPath != (ArtsIpPathData *)0);
      this->_data._ipPath->read(is,this->Header().Version());
      if (is.eof())
        return(is);
      break;

    case artsC_OBJECT_AS_MATRIX:
      if (this->_data._asMatrix) {
        delete(this->_data._asMatrix);
      }
      this->_data._asMatrix = new ArtsAsMatrixData;
      assert(this->_data._asMatrix != (ArtsAsMatrixData *)0);
      this->_data._asMatrix->read(is,this->Header().Version());
      if (is.eof())
        return(is);
      break;

    case artsC_OBJECT_NET:
      if (this->_data._netMatrix) {
        delete(this->_data._netMatrix);
      }
      this->_data._netMatrix = new ArtsNetMatrixData;
      assert(this->_data._netMatrix != (ArtsNetMatrixData *)0);
      this->_data._netMatrix->read(is,this->Header().Version());
      if (is.eof())
        return(is);
      break;

    case artsC_OBJECT_PORT:
      if (this->_data._portTable) {
        delete(this->_data._portTable);
      }
      this->_data._portTable = new ArtsPortTableData;
      assert(this->_data._portTable != (ArtsPortTableData *)0);
      this->_data._portTable->read(is,this->Header().Version());
      if (is.eof())
        return(is);
      break;

    case artsC_OBJECT_SELECTED_PORT:
      if (this->_data._selectedPortTable) {
        delete(this->_data._selectedPortTable);
      }
      this->_data._selectedPortTable = new ArtsSelectedPortTableData;
      assert(this->_data._selectedPortTable != (ArtsSelectedPortTableData *)0);
      this->_data._selectedPortTable->read(is,this->Header().Version());
      break;
      
    case artsC_OBJECT_PORT_MATRIX:
      if (this->_data._portMatrix) {
        delete(this->_data._portMatrix);
      }
      this->_data._portMatrix = new ArtsPortMatrixData;
      assert(this->_data._portMatrix != (ArtsPortMatrixData *)0);
      this->_data._portMatrix->read(is,this->Header().Version());
      if (is.eof())
        return(is);
      break;
      
    case artsC_OBJECT_PROTO:
      if (this->_data._protocolTable) {
        delete(this->_data._protocolTable);
      }
      this->_data._protocolTable = new ArtsProtocolTableData;
      assert(this->_data._protocolTable != (ArtsProtocolTableData *)0);
      this->_data._protocolTable->read(is,this->Header().Version());
      if (is.eof())
        return(is);
      break;

    case artsC_OBJECT_INTERFACE_MATRIX:
      if (this->_data._interfaceMatrix) {
        delete(this->_data._interfaceMatrix);
      }
      this->_data._interfaceMatrix = new ArtsInterfaceMatrixData;
      assert(this->_data._interfaceMatrix != (ArtsInterfaceMatrixData *)0);
      this->_data._interfaceMatrix->read(is,this->Header().Version());
      if (is.eof())
        return(is);
      break;
      
    case artsC_OBJECT_NEXT_HOP:
      if (this->_data._nextHopTable) {
        delete(this->_data._nextHopTable);
      }
      this->_data._nextHopTable = new ArtsNextHopTableData;
      assert(this->_data._nextHopTable != (ArtsNextHopTableData *)0);
      this->_data._nextHopTable->read(is,this->Header().Version());
      if (is.eof())
        return(is);
      break;
      
    default:
      fprintf(stderr,"Unknown ARTS object (id %#x) found.  Skipping data.\n",
              this->Header().Identifier());
      is.seekg(this->Header().DataLength(),ios::cur);
      if (is.eof())
        return(is);
      break;
  }
  return(is);
}

//-------------------------------------------------------------------------
//                       int Arts::read(int fd)                      
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
int Arts::read(int fd)
{
  ArtsAttribute   artsAttribute;
  size_t          attrNum;
  int             rc;
  int             bytesRead = 0;
  
  rc = this->Header().read(fd);
  if (rc <= 0)
    return(rc);
  bytesRead += rc;
  
  for (attrNum = 0; attrNum < this->Header().NumAttributes(); attrNum++) {
    rc = artsAttribute.read(fd);
    if (rc <= 0)
      return(rc);
    bytesRead += rc;
    this->Attributes().push_back(artsAttribute);
  }
  
  switch (this->Header().Identifier()) {
    
    case artsC_OBJECT_IP_PATH:
      if (this->_data._ipPath) {
        delete(this->_data._ipPath);
      }
      this->_data._ipPath = new ArtsIpPathData;
      assert(this->_data._ipPath != (ArtsIpPathData *)0);
      rc = this->IpPathData()->read(fd,this->Header().Version());
      if (rc <= 0)
        return(rc);
      bytesRead += rc;
      break;

    case artsC_OBJECT_AS_MATRIX:
      if (this->_data._asMatrix) {
        delete(this->_data._asMatrix);
      }
      this->_data._asMatrix = new ArtsAsMatrixData;
      assert(this->_data._asMatrix != (ArtsAsMatrixData *)0);
      rc = this->_data._asMatrix->read(fd,this->Header().Version());
      if (rc <= 0)
        return(rc);
      bytesRead += rc;
      break;

    case artsC_OBJECT_NET:
      if (this->_data._netMatrix) {
        delete(this->_data._netMatrix);
      }
      this->_data._netMatrix = new ArtsNetMatrixData;
      assert(this->_data._netMatrix != (ArtsNetMatrixData *)0);
      rc = this->_data._netMatrix->read(fd,this->Header().Version());
      if (rc <= 0)
        return(rc);
      bytesRead += rc;
      break;

    case artsC_OBJECT_PORT:
      if (this->_data._portTable) {
        delete(this->_data._portTable);
      }
      this->_data._portTable = new ArtsPortTableData;
      assert(this->_data._portTable != (ArtsPortTableData *)0);
      rc = this->_data._portTable->read(fd,this->Header().Version());
      if (rc <= 0)
        return(rc);
      bytesRead += rc;
      break;

    case artsC_OBJECT_SELECTED_PORT:
      if (this->_data._selectedPortTable) {
        delete(this->_data._selectedPortTable);
      }
      this->_data._selectedPortTable = new ArtsSelectedPortTableData;
      assert(this->_data._selectedPortTable != (ArtsSelectedPortTableData *)0);
      this->_data._selectedPortTable->read(fd,this->Header().Version());
      if (rc <= 0)
        return(rc);
      bytesRead += rc;
      break;
      
    case artsC_OBJECT_PORT_MATRIX:
      if (this->_data._portMatrix) {
        delete(this->_data._portMatrix);
      }
      this->_data._portMatrix = new ArtsPortMatrixData;
      assert(this->_data._portMatrix != (ArtsPortMatrixData *)0);
      rc = this->_data._portMatrix->read(fd,this->Header().Version());
      if (rc <= 0)
        return(rc);
      bytesRead += rc;
      break;

    case artsC_OBJECT_PROTO:
      if (this->_data._protocolTable) {
        delete(this->_data._protocolTable);
      }
      this->_data._protocolTable = new ArtsProtocolTableData;
      assert(this->_data._protocolTable != (ArtsProtocolTableData *)0);
      rc = this->_data._protocolTable->read(fd,this->Header().Version());
      if (rc <= 0)
        return(rc);
      bytesRead += rc;
      break;

    case artsC_OBJECT_INTERFACE_MATRIX:
      if (this->_data._interfaceMatrix) {
        delete(this->_data._interfaceMatrix);
      }
      this->_data._interfaceMatrix = new ArtsInterfaceMatrixData;
      assert(this->_data._interfaceMatrix != (ArtsInterfaceMatrixData *)0);
      rc = this->_data._interfaceMatrix->read(fd,this->Header().Version());
      if (rc <= 0)
        return(rc);
      bytesRead += rc;
      break;
      
    case artsC_OBJECT_NEXT_HOP:
      if (this->_data._nextHopTable) {
        delete(this->_data._nextHopTable);
      }
      this->_data._nextHopTable = new ArtsNextHopTableData;
      assert(this->_data._nextHopTable != (ArtsNextHopTableData *)0);
      rc = this->_data._nextHopTable->read(fd,this->Header().Version());
      if (rc <= 0)
        return(rc);
      bytesRead += rc;
      break;
      
    default:
      break;
  }
  
  return(bytesRead);
}

//-------------------------------------------------------------------------
//                    ostream& Arts::write(ostream& os)                    
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ostream& Arts::write(ostream& os)
{
  uint16_t   attrNum;

  
  switch (this->Header().Identifier()) {
    case artsC_OBJECT_IP_PATH:
      this->Header().DataLength(this->IpPathData()->Length(this->Header().Version()));
      break;

    case artsC_OBJECT_AS_MATRIX:
      this->Header().DataLength(this->AsMatrixData()->Length(this->Header().Version()));
      break;
      
    case artsC_OBJECT_NET:
      this->Header().DataLength(this->NetMatrixData()->Length(this->Header().Version()));
      break;
      
    case artsC_OBJECT_PORT:
      this->Header().DataLength(this->PortTableData()->Length(this->Header().Version()));
      break;

    case artsC_OBJECT_SELECTED_PORT:
      this->Header().DataLength(this->SelectedPortTableData()->Length(this->Header().Version()));
      break;
      
    case artsC_OBJECT_PORT_MATRIX:
      this->Header().DataLength(this->PortMatrixData()->Length(this->Header().Version()));
      break;
      
    case artsC_OBJECT_PROTO:
      this->Header().DataLength(this->ProtocolTableData()->Length(this->Header().Version()));
      break;
      
    case artsC_OBJECT_INTERFACE_MATRIX:
      this->Header().DataLength(this->InterfaceMatrixData()->Length(this->Header().Version()));
      break;

    case artsC_OBJECT_NEXT_HOP:
      this->Header().DataLength(this->NextHopTableData()->Length(this->Header().Version()));
      break;

    default:
      fprintf(stderr,
              "Unknown ARTS object (id %#x) found {%s:%d}.  Skipping data.\n",
              this->Header().Identifier(),__FILE__,__LINE__);
      break;
  }
      
  this->Header().write(os);
  if (this->Header().NumAttributes() > 0) {
    for (attrNum = 0; attrNum < this->Header().NumAttributes(); attrNum++) {
      this->Attributes()[attrNum].write(os);
    }
  }
  switch (this->Header().Identifier()) {

    case artsC_OBJECT_IP_PATH:
      this->IpPathData()->write(os,this->Header().Version());
      break;

    case artsC_OBJECT_AS_MATRIX:
      this->AsMatrixData()->write(os,this->Header().Version());
      break;

    case artsC_OBJECT_NET:
      this->NetMatrixData()->write(os,this->Header().Version());
      break;

    case artsC_OBJECT_PORT:
      this->PortTableData()->write(os,this->Header().Version());
      break;

    case artsC_OBJECT_SELECTED_PORT:
      this->SelectedPortTableData()->write(os,this->Header().Version());
      break;
      
    case artsC_OBJECT_PORT_MATRIX:
      this->PortMatrixData()->write(os,this->Header().Version());
      break;

    case artsC_OBJECT_PROTO:
      this->ProtocolTableData()->write(os,this->Header().Version());
      break;
      
    case artsC_OBJECT_INTERFACE_MATRIX:
      this->InterfaceMatrixData()->write(os,this->Header().Version());
      break;

    case artsC_OBJECT_NEXT_HOP:
      this->NextHopTableData()->write(os,this->Header().Version());
      break;

    default:
      fprintf(stderr,"Unknown ARTS object (id %#x) {%s:%d}\n",
              this->Header().Identifier(),__FILE__,__LINE__);
      break;
  }
  return(os);
}

//-------------------------------------------------------------------------
//                         int Arts::write(int fd)                         
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
int Arts::write(int fd)
{
  uint16_t   attrNum;
  int        rc;
  int        bytesWritten = 0;
  
  switch (this->Header().Identifier()) {
    case artsC_OBJECT_IP_PATH:
      this->Header().DataLength(this->IpPathData()->Length(this->Header().Version()));
      break;

    case artsC_OBJECT_AS_MATRIX:
      this->Header().DataLength(this->AsMatrixData()->Length(this->Header().Version()));
      break;

    case artsC_OBJECT_NET:
      this->Header().DataLength(this->NetMatrixData()->Length(this->Header().Version()));
      break;

    case artsC_OBJECT_PORT:
      this->Header().DataLength(this->PortTableData()->Length(this->Header().Version()));
      break;

    case artsC_OBJECT_SELECTED_PORT:
      this->Header().DataLength(this->SelectedPortTableData()->Length(this->Header().Version()));
      break;
      
    case artsC_OBJECT_PORT_MATRIX:
      this->Header().DataLength(this->PortMatrixData()->Length(this->Header().Version()));
      break;

    case artsC_OBJECT_PROTO:
      this->Header().DataLength(this->ProtocolTableData()->Length(this->Header().Version()));
      break;
      
    case artsC_OBJECT_INTERFACE_MATRIX:
      this->Header().DataLength(this->InterfaceMatrixData()->Length(this->Header().Version()));
      break;

    case artsC_OBJECT_NEXT_HOP:
      this->Header().DataLength(this->NextHopTableData()->Length(this->Header().Version()));
      break;

    default:
      fprintf(stderr,"Unknown ARTS object (id %#x) {%s:%d}\n",
              this->Header().Identifier(),__FILE__,__LINE__);
      break;
  }
      
  if ((rc = this->Header().write(fd)) < 0) {
    return(-1);
  }
  bytesWritten += rc;
  
  if (this->Header().NumAttributes() > 0) {
    for (attrNum = 0; attrNum < this->Header().NumAttributes(); attrNum++) {
      if ((rc = this->Attributes()[attrNum].write(fd)) < 0) {
        return(-1);
      }
      bytesWritten += rc;
    }
  }
  switch (this->Header().Identifier()) {

    case artsC_OBJECT_IP_PATH:
      if ((rc = this->IpPathData()->write(fd,this->Header().Version())) < 0) {
        return(-1);
      }
      bytesWritten += rc;
      break;

    case artsC_OBJECT_AS_MATRIX:
      rc = this->AsMatrixData()->write(fd,this->Header().Version());
      if (rc < 0) {
        return(-1);
      }
      bytesWritten += rc;
      break;

    case artsC_OBJECT_NET:
      rc = this->NetMatrixData()->write(fd,this->Header().Version());
      if (rc < 0) {
        return(-1);
      }
      bytesWritten += rc;
      break;

    case artsC_OBJECT_PORT:
      rc = this->PortTableData()->write(fd,this->Header().Version());
      if (rc < 0) {
        return(-1);
      }
      bytesWritten += rc;
      break;

    case artsC_OBJECT_SELECTED_PORT:
      rc = this->SelectedPortTableData()->write(fd,this->Header().Version());
      if (rc < 0) {
        return(-1);
      }
      bytesWritten += rc;
      break;
      
    case artsC_OBJECT_PORT_MATRIX:
      rc = this->PortMatrixData()->write(fd,this->Header().Version());
      if (rc < 0) {
        return(-1);
      }
      bytesWritten += rc;
      break;

    case artsC_OBJECT_PROTO:
      rc = this->ProtocolTableData()->write(fd,this->Header().Version());
      if (rc < 0) {
        return(-1);
      }
      bytesWritten += rc;
      break;
      
    case artsC_OBJECT_INTERFACE_MATRIX:
      rc = this->InterfaceMatrixData()->write(fd,this->Header().Version());
      if (rc < 0) {
        return(-1);
      }
      bytesWritten += rc;
      break;

    case artsC_OBJECT_NEXT_HOP:
      rc = this->NextHopTableData()->write(fd,this->Header().Version());
      if (rc < 0) {
        return(-1);
      }
      bytesWritten += rc;
      break;

    default:
      fprintf(stderr,"Unknown ARTS object (id %#x) {%s:%d}\n",
              this->Header().Identifier(),__FILE__,__LINE__);
      break;
  }
  return(bytesWritten);
}

uint32_t Arts::_numObjects = 0;

