//===========================================================================
//  $Name: arts++-0-9-b6 $
//  $Id: ArtsAsMatrixData.cc,v 1.6 1999/01/25 08:37:43 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================
#include <string>

#include "ArtsAsMatrixData.hh"
#include "ArtsPrimitive.hh"

static const string rcsid = "@(#) $Name: arts++-0-9-b6 $ $Id: ArtsAsMatrixData.cc,v 1.6 1999/01/25 08:37:43 dwm Exp $";

//----------------------------------------------------------------------------
//                   ArtsAsMatrixData::ArtsAsMatrixData()                  
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsAsMatrixData::ArtsAsMatrixData()
{
  this->_sampleInterval = 1;
  this->_count = 0;
  this->_totpkts = 0;
  this->_totbytes = 0;
  this->_orphans = 0;

  #ifdef ARTS_DEBUG_CLASS_ALLOC
    ++this->_numObjects;
  #endif
}

//----------------------------------------------------------------------------
//                  ArtsAsMatrixData::~ArtsAsMatrixData()                  
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsAsMatrixData::~ArtsAsMatrixData()
{
  #ifdef ARTS_DEBUG_CLASS_ALLOC
    --this->_numObjects;
  #endif
}

//-------------------------------------------------------------------------
//   istream & ArtsAsMatrixData::read(istream& is, uint8_t version = 0)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
istream & ArtsAsMatrixData::read(istream& is, uint8_t version = 0)
{
  uint32_t           entryNum;
  ArtsPrimitive      ioHelper;
  ArtsAsMatrixEntry  asEntry;
  
  ioHelper.ReadUint16(is,this->_sampleInterval,2);
  ioHelper.ReadUint32(is,this->_count,4);
  ioHelper.ReadUint64(is,this->_totpkts,8);
  ioHelper.ReadUint64(is,this->_totbytes,8);
  ioHelper.ReadUint64(is,this->_orphans,8);
  
  this->_asEntries.reserve(this->_count);

  for (entryNum = 0; entryNum < this->_count; entryNum++) {
    asEntry.read(is,version);
    this->_asEntries.push_back(asEntry);
  }
  return(is);
}

//-------------------------------------------------------------------------
//         int ArtsAsMatrixData::read(int fd, uint8_t version = 0)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
int ArtsAsMatrixData::read(int fd, uint8_t version = 0)
{
  uint32_t           entryNum;
  ArtsPrimitive      ioHelper;
  ArtsAsMatrixEntry  asEntry;
  int                rc;
  int                bytesRead = 0;
  
  if ((rc = ioHelper.ReadUint16(fd,this->_sampleInterval,2)) < 2) {
    return(-1);
  }
  bytesRead += rc;
  
  if ((rc = ioHelper.ReadUint32(fd,this->_count,4)) < 4) {
    return(-1);
  }
  bytesRead += rc;
  
  if ((rc = ioHelper.ReadUint64(fd,this->_totpkts,8)) < 8) {
    return(-1);
  }
  bytesRead += rc;
  
  if ((rc = ioHelper.ReadUint64(fd,this->_totbytes,8)) < 8) {
    return(-1);
  }
  bytesRead += rc;
  
  if ((rc = ioHelper.ReadUint64(fd,this->_orphans,8)) < 8) {
    return(-1);
  }
  bytesRead += rc;
  
  this->_asEntries.reserve(this->_count);

  for (entryNum = 0; entryNum < this->_count; entryNum++) {
    if ((rc = asEntry.read(fd,version)) < 0) {
      return(-1);
    }
    bytesRead += rc;
    this->_asEntries.push_back(asEntry);
  }
  return(bytesRead);
}


//-------------------------------------------------------------------------
//   ostream & ArtsAsMatrixData::write(ostream& os, uint8_t version = 0)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ostream & ArtsAsMatrixData::write(ostream& os, uint8_t version = 0)
{
  uint32_t       entryNum;
  ArtsPrimitive  ioHelper;

  ioHelper.WriteUint16(os,this->_sampleInterval,2);
  this->_count = this->_asEntries.size();
  ioHelper.WriteUint32(os,this->_count,4);
  ioHelper.WriteUint64(os,this->_totpkts,8);
  ioHelper.WriteUint64(os,this->_totbytes,8);
  ioHelper.WriteUint64(os,this->_orphans,8);
  
  for (entryNum = 0; entryNum < this->_count; entryNum++) {
    this->_asEntries[entryNum].write(os);
  }
  return(os);
}

//-------------------------------------------------------------------------
//         int ArtsAsMatrixData::write(int fd, uint8_t version = 0)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
int ArtsAsMatrixData::write(int fd, uint8_t version = 0)
{
  uint32_t       entryNum;
  ArtsPrimitive  ioHelper;
  int            rc;
  int            bytesWritten = 0;
  
  if ((rc = ioHelper.WriteUint16(fd,this->_sampleInterval,2)) < 2) {
    return(-1);
  }
  bytesWritten += rc;

  this->_count = this->_asEntries.size();

  if ((rc = ioHelper.WriteUint32(fd,this->_count,4)) < 4) {
    return(-1);
  }
  bytesWritten += rc;
  
  if ((rc = ioHelper.WriteUint64(fd,this->_totpkts,8)) < 8) {
    return(-1);
  }
  bytesWritten += rc;
  
  if ((rc = ioHelper.WriteUint64(fd,this->_totbytes,8)) < 8) {
    return(-1);
  }
  bytesWritten += rc;
  
  if ((rc = ioHelper.WriteUint64(fd,this->_orphans,8)) < 8) {
    return(-1);
    bytesWritten += rc;
  }
  
  for (entryNum = 0; entryNum < this->_count; entryNum++) {
    if ((rc = this->_asEntries[entryNum].write(fd,version)) < 0) {
      return(-1);
    }
    bytesWritten += rc;
  }
  return(bytesWritten);
}

//-------------------------------------------------------------------------
//     uint32_t ArtsAsMatrixData::Length(uint8_t version = 0) const
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
uint32_t ArtsAsMatrixData::Length(uint8_t version = 0) const
{
  uint32_t  length = 0;

  length += (sizeof(this->_sampleInterval) + sizeof(this->_count) +
             sizeof(this->_totpkts) + sizeof(this->_totbytes) +
             sizeof(this->_orphans));

  vector<ArtsAsMatrixEntry>::const_iterator  asEntryIter;
  for (asEntryIter = this->AsEntries().begin();
       asEntryIter != this->AsEntries().end(); asEntryIter++) {
    length += asEntryIter->Length(version);
  }
  return(length);
}

//-------------------------------------------------------------------------
//    ostream& operator << (ostream& os,                   
//                          const ArtsAsMatrixData & artsAsMatrixData)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ostream& operator << (ostream& os,
                      const ArtsAsMatrixData & artsAsMatrixData)
{
  os << "ASMATRIX OBJECT DATA" << endl;

  os << "\tsample_interval: " << artsAsMatrixData.SampleInterval() << endl;
  os << "\tcount: " << artsAsMatrixData.Count() << endl;
  os << "\ttotpkts: " << artsAsMatrixData.TotalPkts() << endl;
  os << "\ttotbytes: " << artsAsMatrixData.TotalBytes() << endl;
  os << "\torphans: " << artsAsMatrixData.Orphans() << endl;
  
  vector<ArtsAsMatrixEntry>::const_iterator  asEntryIter;

  for (asEntryIter = artsAsMatrixData.AsEntries().begin();
       asEntryIter != artsAsMatrixData.AsEntries().end(); asEntryIter++) {
    os << (*asEntryIter);
  }
  return(os);
}

//-------------------------------------------------------------------------
//               void ArtsAsMatrixData::SortEntriesByBytes()              
//.........................................................................
//  
//-------------------------------------------------------------------------
void ArtsAsMatrixData::SortEntriesByBytes()
{
  sort(this->_asEntries.begin(),this->_asEntries.end(),
       ArtsAsMatrixEntryGreaterBytes());
  return;
}

//-------------------------------------------------------------------------
//               void ArtsAsMatrixData::SortEntriesByPkts()               
//.........................................................................
//  
//-------------------------------------------------------------------------
void ArtsAsMatrixData::SortEntriesByPkts()
{
  sort(this->_asEntries.begin(),this->_asEntries.end(),
       ArtsAsMatrixEntryGreaterPkts());
  return;
}

uint32_t  ArtsAsMatrixData::_numObjects = 0;

