//===========================================================================
//  $Name: arts++-0-9-b6 $
//  $Id: ArtsAsMatrixEntry.cc,v 1.5 1999/01/25 08:37:44 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

extern "C" {
#include <unistd.h>
}

#include <string>

#include "ArtsAsMatrixEntry.hh"
#include "ArtsPrimitive.hh"

static const string rcsid = "@(#) $Name: arts++-0-9-b6 $ $Id: ArtsAsMatrixEntry.cc,v 1.5 1999/01/25 08:37:44 dwm Exp $";

//-------------------------------------------------------------------------
//                  ArtsAsMatrixEntry::ArtsAsMatrixEntry()                 
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ArtsAsMatrixEntry::ArtsAsMatrixEntry()
{
  this->_descriptor = 0;
  this->_src        = 0;
  this->_dst        = 0;
  this->_pkts       = 0;
  this->_bytes      = 0;
  
  #ifdef ARTS_DEBUG_CLASS_ALLOC
    ++this->_numObjects;
  #endif
}

//----------------------------------------------------------------------------
//                 ArtsAsMatrixEntry::~ArtsAsMatrixEntry()                 
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsAsMatrixEntry::~ArtsAsMatrixEntry()
{
  #ifdef ARTS_DEBUG_CLASS_ALLOC
    --this->_numObjects;
  #endif
}

//-------------------------------------------------------------------------
//              uint16_t ArtsAsMatrixEntry::Src(uint16_t src)              
//.........................................................................
//  
//-------------------------------------------------------------------------
uint16_t ArtsAsMatrixEntry::Src(uint16_t src)
{
  this->_src = src;

  if (src > 0xff)
    this->_descriptor = this->_descriptor | 0x01;
  else
    this->_descriptor = this->_descriptor & 0xfe;
  
  return(this->_src);
}


//-------------------------------------------------------------------------
//              uint16_t ArtsAsMatrixEntry::Dst(uint16_t dst)              
//.........................................................................
//  
//-------------------------------------------------------------------------
uint16_t ArtsAsMatrixEntry::Dst(uint16_t dst)
{
  this->_dst = dst;
  if (dst > 0xff)
    this->_descriptor = this->_descriptor | 0x02;
  else
    this->_descriptor = this->_descriptor & 0xfd;
  
  return(this->_dst);
}

//-------------------------------------------------------------------------
//             uint64_t ArtsAsMatrixEntry::Pkts(uint64_t pkts)             
//.........................................................................
//  
//-------------------------------------------------------------------------
uint64_t ArtsAsMatrixEntry::Pkts(uint64_t pkts)
{
  this->_pkts = pkts;

  if (pkts > (uint64_t)0xffffffff) {
    this->_descriptor = (this->_descriptor | 0x1c);
  } else if (pkts > (uint64_t)0xffff) {
    this->_descriptor = (this->_descriptor & 0xe3) | (0x0003 << 2);
  } else if (pkts > (uint64_t)0xff) {
    this->_descriptor = (this->_descriptor & 0xe3) | (0x0001 << 2);
  } else {
    this->_descriptor = (this->_descriptor & 0xe3);
  }
  return(this->_pkts);
}

//-------------------------------------------------------------------------
//           uint64_t ArtsAsMatrixEntry::Bytes(uint64_t bytes)
//.........................................................................
//  
//-------------------------------------------------------------------------
uint64_t ArtsAsMatrixEntry::Bytes(uint64_t bytes)
{
  this->_bytes = bytes;

  if (bytes > (uint64_t)0xffffffff) {
    this->_descriptor = (this->_descriptor | 0xe0);
  } else if (bytes > (uint64_t)0xffff) {
    this->_descriptor = (this->_descriptor & 0x1f) | (0x0003 << 5);
  } else if (bytes > (uint64_t)0xff) {
    this->_descriptor = (this->_descriptor & 0x1f) | (0x0001 << 5);
  } else {
    this->_descriptor = (this->_descriptor & 0x1f);
  }
  return(this->_bytes);
}

//-------------------------------------------------------------------------
//    uint32_t ArtsAsMatrixEntry::Length(uint8_t version = 0) const
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
uint32_t ArtsAsMatrixEntry::Length(uint8_t version = 0) const
{
  uint32_t   length;

  length = (sizeof(this->_descriptor) +
            (this->_descriptor & 0x01) + 1 +
            ((this->_descriptor >> 1) & 0x01) + 1 +
            ((this->_descriptor >> 2) & 0x07) + 1 +
            ((this->_descriptor >> 5) & 0x07) + 1);
  return(length);
}

//-------------------------------------------------------------------------
//   istream& ArtsAsMatrixEntry::read(istream& is, uint8_t version = 0)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
istream& ArtsAsMatrixEntry::read(istream& is, uint8_t version = 0)
{
  ArtsPrimitive  ioHelper;
  uint8_t        bytesize,
                 pktsize,
                 srcsize,
                 dstsize;
  
  is.read(&this->_descriptor,sizeof(this->_descriptor));

  srcsize = (this->_descriptor & 0x01) + 1;
  dstsize  = ((this->_descriptor >> 1) & 0x01) + 1;
  pktsize  = ((this->_descriptor >> 2) & 0x07) + 1;
  bytesize = ((this->_descriptor >> 5) & 0x07) + 1;

  ioHelper.ReadUint16(is,this->_src,srcsize);
  ioHelper.ReadUint16(is,this->_dst,dstsize);
  ioHelper.ReadUint64(is,this->_pkts,pktsize);
  ioHelper.ReadUint64(is,this->_bytes,bytesize);

  return(is);
}

//-------------------------------------------------------------------------
//         int ArtsAsMatrixEntry::read(int fd, uint8_t version = 0)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
int ArtsAsMatrixEntry::read(int fd, uint8_t version = 0)
{
  ArtsPrimitive  ioHelper;
  uint8_t        bytesize,
                 pktsize,
                 srcsize,
                 dstsize;
  int            rc;
  int            bytesRead = 0;
  
  if ((rc = ::read(fd,&this->_descriptor,sizeof(this->_descriptor))) !=
      sizeof(this->_descriptor)) {
    return(-1);
  }
  bytesRead += rc;

  srcsize = (this->_descriptor & 0x01) + 1;
  dstsize  = ((this->_descriptor >> 1) & 0x01) + 1;
  pktsize  = ((this->_descriptor >> 2) & 0x07) + 1;
  bytesize = ((this->_descriptor >> 5) & 0x07) + 1;

  if ((rc = ioHelper.ReadUint16(fd,this->_src,srcsize)) != srcsize) {
    return(-1);
  }
  bytesRead += rc;
  
  if ((rc = ioHelper.ReadUint16(fd,this->_dst,dstsize)) != dstsize) {
    return(-1);
  }
  bytesRead += rc;
  
  if ((rc = ioHelper.ReadUint64(fd,this->_pkts,pktsize)) != pktsize) {
    return(-1);
  }
  bytesRead += rc;
  
  if ((rc = ioHelper.ReadUint64(fd,this->_bytes,bytesize)) != bytesize) {
    return(-1);
  }
  bytesRead += rc;

  return(bytesRead);
}

//-------------------------------------------------------------------------
// ostream& ArtsAsMatrixEntry::write(ostream& os, uint8_t version = 0) const
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ostream & ArtsAsMatrixEntry::write(ostream & os, uint8_t version = 0) const
{
  ArtsPrimitive  ioHelper;
  uint8_t        bytesize,
                 pktsize,
                 srcsize,
                 dstsize;

  os.write(&this->_descriptor,sizeof(this->_descriptor));

  srcsize = (this->_descriptor & 0x01) + 1;
  dstsize  = ((this->_descriptor >> 1) & 0x01) + 1;
  pktsize  = ((this->_descriptor >> 2) & 0x07) + 1;
  bytesize = ((this->_descriptor >> 5) & 0x07) + 1;

  ioHelper.WriteUint16(os,this->_src,srcsize);
  ioHelper.WriteUint16(os,this->_dst,dstsize);
  ioHelper.WriteUint64(os,this->_pkts,pktsize);
  ioHelper.WriteUint64(os,this->_bytes,bytesize);

  return(os);
}

//-------------------------------------------------------------------------
//     int ArtsAsMatrixEntry::write(int fd, uint8_t version = 0) const
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
int ArtsAsMatrixEntry::write(int fd, uint8_t version = 0) const
{
  ArtsPrimitive  ioHelper;
  uint8_t        bytesize,
                 pktsize,
                 srcsize,
                 dstsize;
  int            rc;
  int            bytesWritten = 0;
  
  if ((rc = ::write(fd,&this->_descriptor,sizeof(this->_descriptor))) !=
      sizeof(this->_descriptor)) {
    return(-1);
  }
  bytesWritten += rc;

  srcsize = (this->_descriptor & 0x01) + 1;
  dstsize  = ((this->_descriptor >> 1) & 0x01) + 1;
  pktsize  = ((this->_descriptor >> 2) & 0x07) + 1;
  bytesize = ((this->_descriptor >> 5) & 0x07) + 1;

  if ((rc = ioHelper.WriteUint16(fd,this->_src,srcsize)) != srcsize) {
    return(-1);
  }
  bytesWritten += rc;
  
  if ((rc = ioHelper.WriteUint16(fd,this->_dst,dstsize)) != dstsize) {
    return(-1);
  }
  bytesWritten += rc;
  
  if ((rc = ioHelper.WriteUint64(fd,this->_pkts,pktsize)) != pktsize) {
    return(-1);
  }
  bytesWritten += rc;
  
  if ((rc = ioHelper.WriteUint64(fd,this->_bytes,bytesize)) != bytesize) {
    return(-1);
  }
  bytesWritten += rc;

  return(bytesWritten);
}

//-------------------------------------------------------------------------
//   ostream & operator << (ostream& os,                   
//                          const ArtsAsMatrixEntry & artsAsMatrixEntry) 
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ostream & operator << (ostream& os,
                       const ArtsAsMatrixEntry & artsAsMatrixEntry)
{
  os << "\tAS MATRIX ENTRY" << endl;
  os << "\t\tdescriptor: " << (int)artsAsMatrixEntry.Descriptor() << endl;
  os << "\t\tsrc: " << artsAsMatrixEntry.Src() << endl;
  os << "\t\tdst: " << artsAsMatrixEntry.Dst() << endl;
  os << "\t\tpkts: " << artsAsMatrixEntry.Pkts() << endl;
  os << "\t\tbytes: " << artsAsMatrixEntry.Bytes() << endl;

  return(os);
}

//---------------------------------------------------------------------------
//        bool ArtsAsMatrixEntryGreaterBytes::
//        operator () (const ArtsAsMatrixEntry & asEntry1,
//                     const ArtsAsMatrixEntry & asEntry2) const
//...........................................................................
//
//---------------------------------------------------------------------------
bool ArtsAsMatrixEntryGreaterBytes::
operator () (const ArtsAsMatrixEntry & asEntry1,
             const ArtsAsMatrixEntry & asEntry2) const
{
  return(asEntry1.Bytes() > asEntry2.Bytes());
}

//---------------------------------------------------------------------------
//        bool ArtsAsMatrixEntryGreaterPkts::
//        operator () (const ArtsAsMatrixEntry & asEntry1,
//                     const ArtsAsMatrixEntry & asEntry2) const
//...........................................................................
//
//---------------------------------------------------------------------------
bool ArtsAsMatrixEntryGreaterPkts::
operator () (const ArtsAsMatrixEntry & asEntry1,
             const ArtsAsMatrixEntry & asEntry2) const
{
  return(asEntry1.Pkts() > asEntry2.Pkts());
}

uint32_t ArtsAsMatrixEntry::_numObjects = 0;
