//===========================================================================
//  @(#) $Name: arts++-0-9-b6 $
//  @(#) $Id: ArtsAttribute.cc,v 1.6 1999/01/25 08:37:45 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

extern "C" {
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <time.h>
#include <unistd.h>
#include <stdlib.h>
}

#include <string>
#include <iomanip.h>

#include "ArtsAttribute.hh"

static const string rcsid = "@(#) $Name: arts++-0-9-b6 $ $Id: ArtsAttribute.cc,v 1.6 1999/01/25 08:37:45 dwm Exp $";

//-------------------------------------------------------------------------
//                      ArtsAttribute::ArtsAttribute()                     
//.........................................................................
//  constructor
//-------------------------------------------------------------------------
ArtsAttribute::ArtsAttribute()
{
  this->_identifier = 0;
  this->_format = 0;
  this->_length = 0;

  #ifdef ARTS_DEBUG_CLASS_ALLOC
    ++_numObjects;
  #endif
}
  
//-------------------------------------------------------------------------
//     ArtsAttribute &
//     ArtsAttribute::operator = (const ArtsAttribute & artsAttribute) 
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ArtsAttribute &
ArtsAttribute::operator = (const ArtsAttribute & artsAttribute)
{
  this->_identifier = artsAttribute.Identifier();
  this->_format = artsAttribute.Format();
  this->_length = artsAttribute.Length();

  switch (this->_identifier) {
    case artsC_ATTR_COMMENT:
      this->_value.comment = new string(artsAttribute.Comment());
      break;
    case artsC_ATTR_INTERFACE:
      this->_value.interface = new string(artsAttribute.Interface());
      break;
    case artsC_ATTR_CREATION:
      this->_value.creation = artsAttribute.Creation();
      break;
    case artsC_ATTR_PERIOD:
      memcpy(this->_value.period,artsAttribute.Period(),
             sizeof(this->_value.period[0]) * 2);
      break;
    case artsC_ATTR_HOST:
      this->_value.host = artsAttribute.Host();
      break;
    case artsC_ATTR_IFINDEX:
      this->_value.ifIndex = artsAttribute.IfIndex();
      break;
    default:
      break;
  }
  
  return(*this);
}

//-------------------------------------------------------------------------
//                     ArtsAttribute::~ArtsAttribute()                     
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ArtsAttribute::~ArtsAttribute()
{
  switch (this->_identifier) {
    case artsC_ATTR_COMMENT:
      if (this->_value.comment)
        delete(this->_value.comment);
      break;
    case artsC_ATTR_INTERFACE:
      if (this->_value.interface)
        delete(this->_value.interface);
      break;
    default:
      break;
  }

  #ifdef ARTS_DEBUG_CLASS_ALLOC
    if (_numObjects > 0)
      --_numObjects;
  #endif
}

//-------------------------------------------------------------------------
//             ostream& ArtsAttribute::write(ostream& os) const            
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ostream& ArtsAttribute::write(ostream& os) const
{
  uint32_t     uIntDatum, idAndFormat;
  uint16_t     uShortDatum;
  
  const char*  ptr;

  idAndFormat = (this->_identifier << 8) | this->_format;
  uIntDatum = htonl(idAndFormat);
  os.write(&uIntDatum,sizeof(uIntDatum));

  uIntDatum = htonl(this->_length);
  os.write(&uIntDatum,sizeof(uIntDatum));

  switch (this->_identifier) {
    case artsC_ATTR_COMMENT:
      ptr = this->_value.comment->c_str();
      os.write(ptr,this->_length);
      break;
    case artsC_ATTR_CREATION:
      uIntDatum = htonl(this->_value.creation);
      os.write(&uIntDatum,sizeof(uIntDatum));
      break;
    case artsC_ATTR_PERIOD:
      uIntDatum = htonl(this->_value.period[0]);
      os.write(&uIntDatum,sizeof(uIntDatum));
      uIntDatum = htonl(this->_value.period[1]);
      os.write(&uIntDatum,sizeof(uIntDatum));
      break;
    case artsC_ATTR_HOST:
      os.write(&this->_value.host,sizeof(this->_value.host));
      break;
    case artsC_ATTR_INTERFACE:
      ptr = this->_value.interface->c_str();
      os.write(ptr,this->_length);
      break;
    case artsC_ATTR_IFINDEX:
      uShortDatum = htons(this->_value.ifIndex);
      os.write(&uShortDatum,sizeof(uShortDatum));
      break;
    default:
      break;
  }
  return(os);
}

//-------------------------------------------------------------------------
//                  int ArtsAttribute::write(int fd) const                 
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
int ArtsAttribute::write(int fd) const
{
  uint32_t     uIntDatum, idAndFormat;
  uint16_t     uShortDatum;
  const char*  ptr;
  int          rc;
  int          bytesWritten = 0;
  
  idAndFormat = (this->_identifier << 8) | this->_format;
  uIntDatum = htonl(idAndFormat);
  if ((rc = ::write(fd,&uIntDatum,sizeof(uIntDatum))) <
      (int)sizeof(uIntDatum)) {
    return(-1);
  }
  bytesWritten += rc;

  uIntDatum = htonl(this->_length);
  if ((rc = ::write(fd,&uIntDatum,sizeof(uIntDatum))) <
      (int)sizeof(uIntDatum)) {
    return(-1);
  }
  bytesWritten += rc;

  switch (this->_identifier) {
    case artsC_ATTR_COMMENT:
      ptr = this->_value.comment->c_str();
      if ((rc = ::write(fd,ptr,this->_length)) != (int)this->_length) {
        return(-1);
      }
      bytesWritten += rc;
      break;
    case artsC_ATTR_CREATION:
      uIntDatum = htonl(this->_value.creation);
      if ((rc = ::write(fd,&uIntDatum,sizeof(uIntDatum))) !=
          sizeof(uIntDatum)) {
        return(-1);
      }
      bytesWritten += rc;
      break;
    case artsC_ATTR_PERIOD:
      uIntDatum = htonl(this->_value.period[0]);
      if ((rc = ::write(fd,&uIntDatum,sizeof(uIntDatum))) !=
          sizeof(uIntDatum)) {
        return(-1);
      }
      bytesWritten += rc;
      uIntDatum = htonl(this->_value.period[1]);
      if ((rc = ::write(fd,&uIntDatum,sizeof(uIntDatum))) !=
          sizeof(uIntDatum)) {
        return(-1);
      }
      bytesWritten += rc;
      break;
    case artsC_ATTR_HOST:
      if ((rc = ::write(fd,&this->_value.host,sizeof(this->_value.host))) !=
          sizeof(this->_value.host)) {
        return(-1);
      }
      bytesWritten += rc;
      break;
    case artsC_ATTR_INTERFACE:
      ptr = this->_value.interface->c_str();
      if ((rc = ::write(fd,ptr,this->_length)) != (int)this->_length) {
        return(-1);
      }
      bytesWritten += rc;
      break;
    case artsC_ATTR_IFINDEX:
      uShortDatum = htons(this->_value.ifIndex);
      if ((rc = ::write(fd,&uShortDatum,sizeof(uShortDatum))) !=
          sizeof(uShortDatum)) {
        return(-1);
      }
      bytesWritten += rc;
      break;
    default:
      break;
  }
  return(bytesWritten);
}

//-------------------------------------------------------------------------
//                istream& ArtsAttribute::read(istream& is)                
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
istream& ArtsAttribute::read(istream& is)
{
  uint32_t  uIntDatum, idAndFormat;
  char*     ptr;
  
  is.read(&uIntDatum,sizeof(uIntDatum));
  idAndFormat = ntohl(uIntDatum);
  this->_identifier = idAndFormat >> 8;
  this->_format = idAndFormat & 0xff;

  is.read(&uIntDatum,sizeof(uIntDatum));
  this->_length = ntohl(uIntDatum);

  switch (this->_identifier) {
    case artsC_ATTR_COMMENT:
      ptr = (char *)malloc(this->_length);
      assert(ptr);
      is.read(ptr,this->_length);
      if (this->_value.comment)
        delete(this->_value.comment);
      this->_value.comment = new string(ptr);
      free(ptr);
      break;
    case artsC_ATTR_CREATION:
      is.read(&uIntDatum,sizeof(uIntDatum));
      this->_value.creation = ntohl(uIntDatum);
      break;
    case artsC_ATTR_PERIOD:
      is.read(&uIntDatum,sizeof(uIntDatum));
      this->_value.period[0] = htonl(uIntDatum);
      is.read(&uIntDatum,sizeof(uIntDatum));
      this->_value.period[1] = htonl(uIntDatum);
      break;
    case artsC_ATTR_HOST:
      is.read(&this->_value.host,sizeof(this->_value.host));
      break;
    case artsC_ATTR_INTERFACE:
      ptr = (char *)malloc(this->_length);
      assert(ptr);
      is.read(ptr,this->_length);
      if (this->_value.interface)
        delete(this->_value.interface);
      this->_value.interface = new string(ptr);
      free(ptr);
      break;
    case artsC_ATTR_IFINDEX:
      is.read(&this->_value.ifIndex,sizeof(this->_value.ifIndex));
      this->_value.ifIndex = ntohs(this->_value.ifIndex);
      break;
    default:
      break;
  }

  return(is);
}

//-------------------------------------------------------------------------
//                     int ArtsAttribute::read(int fd)                     
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
int ArtsAttribute::read(int fd)
{
  uint32_t  uIntDatum, idAndFormat;
  char*     ptr;
  int       rc;
  int       bytesRead = 0;
  
  rc = ::read(fd,&uIntDatum,sizeof(uIntDatum));
  if (rc <= 0)
    return(rc);
  bytesRead += rc;
  idAndFormat = ntohl(uIntDatum);
  this->_identifier = idAndFormat >> 8;
  this->_format = idAndFormat & 0xff;

  rc = ::read(fd,&uIntDatum,sizeof(uIntDatum));
  if (rc <= 0)
    return(rc);
  bytesRead += rc;
  
  this->_length = ntohl(uIntDatum);

  switch (this->_identifier) {
    case artsC_ATTR_COMMENT:
      ptr = (char *)malloc(this->_length);
      assert(ptr);
      rc = ::read(fd,ptr,this->_length);
      if (rc <= 0)
        return(rc);
      bytesRead += rc;
      if (this->_value.comment)
        delete(this->_value.comment);
      this->_value.comment = new string(ptr);
      free(ptr);
      break;
    case artsC_ATTR_CREATION:
      rc = ::read(fd,&uIntDatum,sizeof(uIntDatum));
      if (rc <= 0)
        return(rc);
      bytesRead += rc;
      this->_value.creation = ntohl(uIntDatum);
      break;
    case artsC_ATTR_PERIOD:
      rc = ::read(fd,&uIntDatum,sizeof(uIntDatum));
      if (rc <= 0)
        return(rc);
      bytesRead += rc;
      this->_value.period[0] = htonl(uIntDatum);
      rc = ::read(fd,&uIntDatum,sizeof(uIntDatum));
      if (rc <= 0)
        return(rc);
      bytesRead += rc;
      this->_value.period[1] = htonl(uIntDatum);
      break;
    case artsC_ATTR_HOST:
      rc = ::read(fd,&this->_value.host,sizeof(this->_value.host));
      if (rc <= 0)
        return(rc);
      bytesRead += rc;
      break;
    case artsC_ATTR_INTERFACE:
      ptr = (char *)malloc(this->_length);
      assert(ptr);
      rc = ::read(fd,ptr,this->_length);
      if (rc <= 0)
        return(rc);
      bytesRead += rc;
      if (this->_value.interface)
        delete(this->_value.interface);
      this->_value.interface = new string(ptr);
      free(ptr);
      break;
    case artsC_ATTR_IFINDEX:
      rc = ::read(fd,&this->_value.ifIndex,sizeof(this->_value.ifIndex));
      if (rc <= 0)
        return(rc);
      this->_value.ifIndex = ntohs(this->_value.ifIndex);
      bytesRead += rc;
      break;
    default:
      break;
  }

  return(bytesRead);
}

//-------------------------------------------------------------------------
//    ostream & operator << (ostream &os, ArtsAttribute & artsAttribute)   
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ostream & operator << (ostream &os, ArtsAttribute & artsAttribute)
{
  struct in_addr  inAddr;
  time_t          timestamp;
  struct tm      *Tm;
  
  os << "ATTRIBUTE" << endl;
  switch (artsAttribute.Identifier()) {
    case artsC_ATTR_COMMENT:
      break;
    case artsC_ATTR_CREATION:
      timestamp = artsAttribute.Creation();
      Tm = localtime(&timestamp);
      os << setiosflags(ios::internal);
      os << "\tcreation: " << setfill('0') << setw(2) << Tm->tm_mon+1 << "/"
         << setw(2) << Tm->tm_mday << "/" << setw(4) << Tm->tm_year+1900
         << " " << setw(2) << Tm->tm_hour << ":" << setw(2)
         << Tm->tm_min << ":" << setw(2) << Tm->tm_sec << " ("
         << hex << (int)(artsAttribute.Creation()) << ")" << endl;
      os << setfill(' ');
      break;
    case artsC_ATTR_PERIOD:
      os << "\tperiod: " << dec << artsAttribute.Period()[0] << " "
         << artsAttribute.Period()[1] << endl;
      break;
    case artsC_ATTR_HOST:
      inAddr.s_addr = artsAttribute.Host();
      os << "\thost: " << inet_ntoa(inAddr) << endl;
      break;
    case artsC_ATTR_INTERFACE:
      os << "\tinterface: " << artsAttribute.Interface() << endl;
      break;
    case artsC_ATTR_IFINDEX:
      os << "\tifIndex: " << artsAttribute.IfIndex() << endl;
      break;
    default:
      break;
  }
  return(os);
}

uint32_t  ArtsAttribute::_numObjects = 0;
