//===========================================================================
//  $Name: arts++-0-9-b6 $
//  $Id: ArtsBgp4Attribute.cc,v 1.4 1999/01/25 08:37:45 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

extern "C" {
#include <assert.h>
#include <string.h>
}

#include <string>

#include "ArtsBgp4Attribute.hh"

static const string rcsid = "@(#) $Name: arts++-0-9-b6 $ $Id: ArtsBgp4Attribute.cc,v 1.4 1999/01/25 08:37:45 dwm Exp $";

//-------------------------------------------------------------------------
//                  ArtsBgp4Attribute::ArtsBgp4Attribute()                 
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ArtsBgp4Attribute::ArtsBgp4Attribute()
{
  #ifdef ARTS_DEBUG_CLASS_ALLOC
    cerr << "ArtsBgp4Attribute constructor called for " << (void *)this
         << endl;
  #endif

  this->_flags = 0;
  this->_type = 0;
  this->_length = 0;
  memset(&this->_value,0,sizeof(this->_value));

  #ifdef ARTS_DEBUG_CLASS_ALLOC
    cerr << "ArtsBgp4Attribute constructor done for " << (void *)this
         << endl;
  #endif
}

//-------------------------------------------------------------------------
//                 ArtsBgp4Attribute::~ArtsBgp4Attribute()                 
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ArtsBgp4Attribute::~ArtsBgp4Attribute()
{
  #ifdef ARTS_DEBUG_CLASS_ALLOC
    cerr << "ArtsBgp4Attribute destructor called for " << (void *)this
         << endl;
  #endif

  this->DeleteValue();

  #ifdef ARTS_DEBUG_CLASS_ALLOC
    cerr << "ArtsBgp4Attribute destructor done for " << (void *)this
         << endl;
  #endif
}

//-------------------------------------------------------------------------
//                 uint8_t ArtsBgp4Attribute::Flags() const                
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
uint8_t ArtsBgp4Attribute::Flags() const
{
  return(this->_flags);
}

//-------------------------------------------------------------------------
//             uint8_t ArtsBgp4Attribute::Flags(uint8_t flags)             
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
uint8_t ArtsBgp4Attribute::Flags(uint8_t flags)
{
  this->_flags = flags;
  return(this->_flags);
}

//-------------------------------------------------------------------------
//                 uint8_t ArtsBgp4Attribute::Type() const                 
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
uint8_t ArtsBgp4Attribute::Type() const
{
  return(this->_type);
}

//-------------------------------------------------------------------------
//            uint8_t ArtsBgp4Attribute::Type(uint8_t attrType)            
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
uint8_t ArtsBgp4Attribute::Type(uint8_t attrType)
{
  this->_type = attrType;
  return(this->_type);
}

//-------------------------------------------------------------------------
//                uint16_t ArtsBgp4Attribute::Length() const               
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
uint16_t ArtsBgp4Attribute::Length() const
{
  return(this->_length);
}

//-------------------------------------------------------------------------
//           uint16_t ArtsBgp4Attribute::Length(uint16_t length)           
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
uint16_t ArtsBgp4Attribute::Length(uint16_t length)
{
  this->_length = length;
  return(this->_length);
}

//-------------------------------------------------------------------------
//                uint8_t ArtsBgp4Attribute::Origin() const
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
uint8_t ArtsBgp4Attribute::Origin() const
{
  assert(this->_type == Bgp4_Attribute_Origin);
  return(this->_value._origin);
}

//-------------------------------------------------------------------------
//            uint8_t ArtsBgp4Attribute::Origin(uint8_t origin)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
uint8_t ArtsBgp4Attribute::Origin(uint8_t origin)
{
  assert(this->_type == Bgp4_Attribute_Origin);
  this->_value._origin = origin;
  return(this->_value._origin);
}

//-------------------------------------------------------------------------
//    const ArtsBgp4AsPathAttribute * ArtsBgp4Attribute::AsPath() const
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
const ArtsBgp4AsPathAttribute * ArtsBgp4Attribute::AsPath() const
{
  assert(this->_type == Bgp4_Attribute_AsPath);
  return(this->_value._asPath);
}

//-------------------------------------------------------------------------
//    const ArtsBgp4AsPathAttribute *
//    ArtsBgp4Attribute::AsPath(const ArtsBgp4AsPathAttribute & asPath)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
const ArtsBgp4AsPathAttribute *
ArtsBgp4Attribute::AsPath(const ArtsBgp4AsPathAttribute & asPath)
{
  this->DeleteValue();
  this->_type = Bgp4_Attribute_AsPath;
  this->_value._asPath = new ArtsBgp4AsPathAttribute(asPath);
  return(this->_value._asPath);
}

//-------------------------------------------------------------------------
//        const ArtsBgp4Prefix * ArtsBgp4Attribute::NextHop() const
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
const ArtsBgp4Prefix * ArtsBgp4Attribute::NextHop() const
{
  assert(this->_type == Bgp4_Attribute_NextHop);
  return(this->_value._nextHop);
}

//-------------------------------------------------------------------------
//    const ArtsBgp4Prefix *
//          ArtsBgp4Attribute::NextHop(const ArtsBgp4Prefix & nextHop)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
const ArtsBgp4Prefix *
ArtsBgp4Attribute::NextHop(const ArtsBgp4Prefix & nextHop)
{
  this->DeleteValue();
  this->_type = Bgp4_Attribute_NextHop;
  this->_value._nextHop = new ArtsBgp4Prefix(nextHop);
  return(this->_value._nextHop);
}

//-------------------------------------------------------------------------
//                 uint32_t ArtsBgp4Attribute::MED() const;                
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
uint32_t ArtsBgp4Attribute::MED() const
{
  assert(this->_type == Bgp4_Attribute_MultiExitDisc);
  return(this->_value._MED);
}

//-------------------------------------------------------------------------
//              uint32_t ArtsBgp4Attribute::MED(uint32_t med)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
uint32_t ArtsBgp4Attribute::MED(uint32_t med)
{
  assert(this->_type == Bgp4_Attribute_MultiExitDisc);
  this->_value._MED = med;
  return(this->_value._MED);
}

//-------------------------------------------------------------------------
//              uint32_t ArtsBgp4Attribute::LocalPref() const
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
uint32_t ArtsBgp4Attribute::LocalPref() const
{
  assert(this->_type == Bgp4_Attribute_LocalPref);
  return(this->_value._localPref);
}

//-------------------------------------------------------------------------
//        uint32_t ArtsBgp4Attribute::LocalPref(uint32_t localPref)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
uint32_t ArtsBgp4Attribute::LocalPref(uint32_t localPref)
{
  assert(this->_type == Bgp4_Attribute_LocalPref);
  this->_value._localPref = localPref;
  return(this->_value._localPref);
}

//-------------------------------------------------------------------------
//               const ArtsBgp4AggregatorAttribute *
//               ArtsBgp4Attribute::Aggregator() const
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
const ArtsBgp4AggregatorAttribute *
ArtsBgp4Attribute::Aggregator() const
{
  assert(this->_type == Bgp4_Attribute_Aggregator);
  return(this->_value._aggregator);
}

//-------------------------------------------------------------------------
//  const ArtsBgp4AggregatorAttribute *
//  ArtsBgp4Attribute::Aggregator(const ArtsBgp4AggregatorAttribute &
//                                aggregator)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
const ArtsBgp4AggregatorAttribute *
ArtsBgp4Attribute::Aggregator(const ArtsBgp4AggregatorAttribute & aggregator)
{
  this->DeleteValue();
  this->_type = Bgp4_Attribute_Aggregator;
  this->_value._aggregator = new ArtsBgp4AggregatorAttribute(aggregator);
  return(this->_value._aggregator);
}

//-------------------------------------------------------------------------
//      const deque<uint32_t> * ArtsBgp4Attribute::Community() const
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
const deque<uint32_t> * ArtsBgp4Attribute::Community() const
{
  assert(this->_type == Bgp4_Attribute_Community);
  return(this->_value._community);
}

//-------------------------------------------------------------------------
//     const deque<uint32_t> *
//     ArtsBgp4Attribute::Community(const deque<uint32_t> & community) 
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
const deque<uint32_t> *
ArtsBgp4Attribute::Community(const deque<uint32_t> & community)
{
  this->DeleteValue();
  this->_type = Bgp4_Attribute_Community;
  this->_value._community = new deque<uint32_t>(community);
  return(this->_value._community);
}


//-------------------------------------------------------------------------
//        const ArtsBgpDPAttribute * ArtsBgp4Attribute::DP() const
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
const ArtsBgp4DPAttribute * ArtsBgp4Attribute::DP() const
{
  assert(this->_type == Bgp4_Attribute_DPA);
  return(this->_value._dp);
}

//-------------------------------------------------------------------------
//         const ArtsBgp4DPAttribute *
//         ArtsBgp4Attribute::DP(const ArtsBgp4DPAttribute & dp)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
const ArtsBgp4DPAttribute *
ArtsBgp4Attribute::DP(const ArtsBgp4DPAttribute & dp)
{
  this->DeleteValue();
  this->_type = Bgp4_Attribute_DPA;
  this->_value._dp = new ArtsBgp4DPAttribute(dp);  
  return(this->_value._dp);
}

//-------------------------------------------------------------------------
//                  void ArtsBgp4Attribute::DeleteValue()
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
void ArtsBgp4Attribute::DeleteValue()
{
  switch (this->_type) {
    case Bgp4_Attribute_AsPath:
      if (this->_value._asPath != (ArtsBgp4AsPathAttribute *)NULL) {
        delete(this->_value._asPath);
      }
      this->_value._asPath = (ArtsBgp4AsPathAttribute *)NULL;
      break;
    case Bgp4_Attribute_NextHop:
      if (this->_value._nextHop != (ArtsBgp4Prefix *)NULL) {
        delete(this->_value._nextHop);
      }
      this->_value._nextHop = (ArtsBgp4Prefix *)NULL;
      break;
    case Bgp4_Attribute_Aggregator:
      if (this->_value._aggregator != (ArtsBgp4AggregatorAttribute *)NULL) {
        delete(this->_value._aggregator);
      }
      this->_value._aggregator = (ArtsBgp4AggregatorAttribute *)NULL;
      break;
    case Bgp4_Attribute_Community:
      if (this->_value._community != (deque<uint32_t> *)NULL) {
        delete(this->_value._community);
      }
      this->_value._community = (deque<uint32_t> *)NULL;
      break;
    case Bgp4_Attribute_DPA:
      if (this->_value._dp != (ArtsBgp4DPAttribute *)NULL) {
        delete(this->_value._dp);
      }
      this->_value._dp = (ArtsBgp4DPAttribute *)NULL;
      break;
    default:
      break;
  }
  return;
}


