//===========================================================================
//  $Name: arts++-0-9-b6 $
//  $Id: ArtsInterfaceMatrixData.cc,v 1.3 1999/01/25 08:37:54 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================
#include <string>

#include "ArtsInterfaceMatrixData.hh"
#include "ArtsPrimitive.hh"

template class vector<ArtsInterfaceMatrixEntry>;

static const string rcsid = "@(#) $Name: arts++-0-9-b6 $ $Id: ArtsInterfaceMatrixData.cc,v 1.3 1999/01/25 08:37:54 dwm Exp $";

//----------------------------------------------------------------------------
//            ArtsInterfaceMatrixData::ArtsInterfaceMatrixData()           
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsInterfaceMatrixData::ArtsInterfaceMatrixData()
{
  this->_sampleInterval = 1;
  this->_count = 0;
  this->_totpkts = 0;
  this->_totbytes = 0;
  this->_orphans = 0;

  #ifdef ARTS_DEBUG_CLASS_ALLOC
    ++this->_numObjects;
  #endif
}


//----------------------------------------------------------------------------
//           ArtsInterfaceMatrixData::~ArtsInterfaceMatrixData()           
//............................................................................
//  
//----------------------------------------------------------------------------
ArtsInterfaceMatrixData::~ArtsInterfaceMatrixData()
{
  #ifdef ARTS_DEBUG_CLASS_ALLOC
    --this->_numObjects;
  #endif
}

//-------------------------------------------------------------------------
// istream & ArtsInterfaceMatrixData::read(istream& is, uint8_t version = 0)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
istream & ArtsInterfaceMatrixData::read(istream& is, uint8_t version = 0)
{
  uint32_t                  entryNum;
  ArtsPrimitive             ioHelper;
  ArtsInterfaceMatrixEntry  interfaceEntry;
  
  ioHelper.ReadUint16(is,this->_sampleInterval,2);
  ioHelper.ReadUint32(is,this->_count,4);
  ioHelper.ReadUint64(is,this->_totpkts,8);
  ioHelper.ReadUint64(is,this->_totbytes,8);
  ioHelper.ReadUint64(is,this->_orphans,8);
  
  this->_interfaceEntries.reserve(this->_count);

  for (entryNum = 0; entryNum < this->_count; entryNum++) {
    interfaceEntry.read(is,version);
    this->_interfaceEntries.push_back(interfaceEntry);
  }
  return(is);
}

//-------------------------------------------------------------------------
//     int ArtsInterfaceMatrixData::read(int fd, uint8_t version = 0)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
int ArtsInterfaceMatrixData::read(int fd, uint8_t version = 0)
{
  uint32_t                  entryNum;
  ArtsPrimitive             ioHelper;
  ArtsInterfaceMatrixEntry  interfaceEntry;
  int                       rc;
  int                       bytesRead = 0;
  
  if ((rc = ioHelper.ReadUint16(fd,this->_sampleInterval,2)) < 2) {
    return(-1);
  }
  bytesRead += rc;
  
  if ((rc = ioHelper.ReadUint32(fd,this->_count,4)) < 4) {
    return(-1);
  }
  bytesRead += rc;
  
  if ((rc = ioHelper.ReadUint64(fd,this->_totpkts,8)) < 8) {
    return(-1);
  }
  bytesRead += rc;
  
  if ((rc = ioHelper.ReadUint64(fd,this->_totbytes,8)) < 8) {
    return(-1);
  }
  bytesRead += rc;
  
  if ((rc = ioHelper.ReadUint64(fd,this->_orphans,8)) < 8) {
    return(-1);
  }
  bytesRead += rc;
  
  this->_interfaceEntries.reserve(this->_count);

  for (entryNum = 0; entryNum < this->_count; entryNum++) {
    if ((rc = interfaceEntry.read(fd,version)) < 0) {
      return(-1);
    }
    bytesRead += rc;
    this->_interfaceEntries.push_back(interfaceEntry);
  }
  return(bytesRead);
}

//-------------------------------------------------------------------------
// ostream & ArtsInterfaceMatrixData::write(ostream& os, uint8_t version = 0)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ostream & ArtsInterfaceMatrixData::write(ostream& os, uint8_t version = 0)
{
  uint32_t       entryNum;
  ArtsPrimitive  ioHelper;

  ioHelper.WriteUint16(os,this->_sampleInterval,2);
  this->_count = this->_interfaceEntries.size();
  ioHelper.WriteUint32(os,this->_count,4);
  ioHelper.WriteUint64(os,this->_totpkts,8);
  ioHelper.WriteUint64(os,this->_totbytes,8);
  ioHelper.WriteUint64(os,this->_orphans,8);
  
  for (entryNum = 0; entryNum < this->_count; entryNum++) {
    this->_interfaceEntries[entryNum].write(os);
  }
  return(os);
}

//-------------------------------------------------------------------------
//     int ArtsInterfaceMatrixData::write(int fd, uint8_t version = 0)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
int ArtsInterfaceMatrixData::write(int fd, uint8_t version = 0)
{
  uint32_t       entryNum;
  ArtsPrimitive  ioHelper;
  int            rc;
  int            bytesWritten = 0;
  
  if ((rc = ioHelper.WriteUint16(fd,this->_sampleInterval,2)) < 2) {
    return(-1);
  }
  bytesWritten += rc;

  this->_count = this->_interfaceEntries.size();

  if ((rc = ioHelper.WriteUint32(fd,this->_count,4)) < 4) {
    return(-1);
  }
  bytesWritten += rc;
  
  if ((rc = ioHelper.WriteUint64(fd,this->_totpkts,8)) < 8) {
    return(-1);
  }
  bytesWritten += rc;
  
  if ((rc = ioHelper.WriteUint64(fd,this->_totbytes,8)) < 8) {
    return(-1);
  }
  bytesWritten += rc;
  
  if ((rc = ioHelper.WriteUint64(fd,this->_orphans,8)) < 8) {
    return(-1);
    bytesWritten += rc;
  }
  
  for (entryNum = 0; entryNum < this->_count; entryNum++) {
    if ((rc = this->_interfaceEntries[entryNum].write(fd,version)) < 0) {
      return(-1);
    }
    bytesWritten += rc;
  }
  return(bytesWritten);
}

//-------------------------------------------------------------------------
//   uint32_t ArtsInterfaceMatrixData::Length(uint8_t version = 0) const
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
uint32_t ArtsInterfaceMatrixData::Length(uint8_t version = 0) const
{
  uint32_t  length = 0;

  length += (sizeof(this->_sampleInterval) + sizeof(this->_count) +
             sizeof(this->_totpkts) + sizeof(this->_totbytes) +
             sizeof(this->_orphans));

  vector<ArtsInterfaceMatrixEntry>::const_iterator  interfaceEntryIter;
  for (interfaceEntryIter = this->InterfaceEntries().begin();
       interfaceEntryIter != this->InterfaceEntries().end();
       interfaceEntryIter++) {
    length += interfaceEntryIter->Length(version);
  }
  return(length);
}

//-------------------------------------------------------------------------
//  ostream &
//  operator << (ostream& os,                   
//               const ArtsInterfaceMatrixData & artsInterfaceMatrixData)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ostream & operator << (ostream& os,
                       const ArtsInterfaceMatrixData & artsInterfaceMatrixData)
{
  os << "INTERFACEMATRIX OBJECT DATA" << endl;

  os << "\tsample_interval: " << artsInterfaceMatrixData.SampleInterval()
     << endl;
  os << "\tcount: " << artsInterfaceMatrixData.Count() << endl;
  os << "\ttotpkts: " << artsInterfaceMatrixData.TotalPkts() << endl;
  os << "\ttotbytes: " << artsInterfaceMatrixData.TotalBytes() << endl;
  os << "\torphans: " << artsInterfaceMatrixData.Orphans() << endl;
  
  vector<ArtsInterfaceMatrixEntry>::const_iterator  interfaceEntryIter;

  for (interfaceEntryIter = artsInterfaceMatrixData.InterfaceEntries().begin();
       interfaceEntryIter != artsInterfaceMatrixData.InterfaceEntries().end();
       interfaceEntryIter++) {
    os << (*interfaceEntryIter);
  }
  return(os);
}

//-------------------------------------------------------------------------
//          void ArtsInterfaceMatrixData::SortEntriesByBytes()
//.........................................................................
//  
//-------------------------------------------------------------------------
void ArtsInterfaceMatrixData::SortEntriesByBytes()
{
  sort(this->_interfaceEntries.begin(),this->_interfaceEntries.end(),
       ArtsInterfaceMatrixEntryGreaterBytes());
  return;
}

//-------------------------------------------------------------------------
//           void ArtsInterfaceMatrixData::SortEntriesByPkts()
//.........................................................................
//  
//-------------------------------------------------------------------------
void ArtsInterfaceMatrixData::SortEntriesByPkts()
{
  sort(this->_interfaceEntries.begin(),this->_interfaceEntries.end(),
       ArtsInterfaceMatrixEntryGreaterPkts());
  return;
}

uint32_t  ArtsInterfaceMatrixData::_numObjects = 0;
