//===========================================================================
// @(#) $Name: arts++-0-9-b6 $
// @(#) $Id: ArtsIpPathEntry.cc,v 1.3 1999/01/25 08:38:09 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

extern "C" {
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
}

#include <string>

#include "ArtsIpPathEntry.hh"

static const string rcsid = "@(#) $Name: arts++-0-9-b6 $ $Id: ArtsIpPathEntry.cc,v 1.3 1999/01/25 08:38:09 dwm Exp $";

//------------------------------------------------------------------------
//                 ArtsIpPathEntry::ArtsIpPathEntry()
//------------------------------------------------------------------------
ArtsIpPathEntry::ArtsIpPathEntry()
{
  this->_hopNum = 0;
  this->_ipAddr = 0;
  this->_bytes = sizeof(this->_hopNum) + sizeof(this->_ipAddr);

  #ifdef ARTS_DEBUG_CLASS_ALLOC
    ++this->_numObjects;
  #endif
  
}

//------------------------------------------------------------------------
//  ArtsIpPathEntry::ArtsIpPathEntry(ipv4addr_t ipAddr, uint8_t hopNum)
//------------------------------------------------------------------------
ArtsIpPathEntry::ArtsIpPathEntry(ipv4addr_t ipAddr, uint8_t hopNum)
{
  this->_ipAddr = ipAddr;
  this->_hopNum = hopNum;
  this->_bytes = sizeof(this->_hopNum) + sizeof(this->_ipAddr);

  #ifdef ARTS_DEBUG_CLASS_ALLOC
    ++this->_numObjects;
  #endif
}

//-------------------------------------------------------------------------
//                   ArtsIpPathEntry::~ArtsIpPathEntry()                   
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
ArtsIpPathEntry::~ArtsIpPathEntry()
{
  #ifdef ARTS_DEBUG_CLASS_ALLOC
    --this->_numObjects;
  #endif
}

//------------------------------------------------------------------------
//
//------------------------------------------------------------------------
ArtsIpPathEntry &
ArtsIpPathEntry::operator = (const ArtsIpPathEntry& artsIpPathEntry)
{
  this->_ipAddr = artsIpPathEntry.IpAddr();
  this->_hopNum = artsIpPathEntry.HopNum();
  this->_bytes = artsIpPathEntry.Length();
  return(*this);
}


//------------------------------------------------------------------------
//    istream& ArtsIpPathEntry::read(istream& is, uint8_t version = 0)
//------------------------------------------------------------------------
istream& ArtsIpPathEntry::read(istream& is, uint8_t version = 0)
{
  is.read(&this->_hopNum,sizeof(this->_hopNum));
  return(is.read(&this->_ipAddr,sizeof(this->_ipAddr)));
}

//-------------------------------------------------------------------------
//        int ArtsIpPathEntry::read(int fd, uint8_t version = 0)
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
int ArtsIpPathEntry::read(int fd, uint8_t version = 0)
{
  int rc;
  int bytesRead = 0;
  
  rc = ::read(fd,&this->_hopNum,sizeof(this->_hopNum));
  if (rc <= 0)
    return(rc);
  bytesRead += rc;
  rc = ::read(fd,&this->_ipAddr,sizeof(this->_ipAddr));
  if (rc <= 0)
    return(rc);
  bytesRead += rc;

  return(bytesRead);
}

//------------------------------------------------------------------------
// ostream& ArtsIpPathEntry::write(ostream& os, uint8_t version = 0) const
//------------------------------------------------------------------------
ostream& ArtsIpPathEntry::write(ostream& os, uint8_t version = 0) const
{
  os.write(&this->_hopNum,sizeof(this->_hopNum));
  return(os.write(&this->_ipAddr,sizeof(this->_ipAddr)));
}

//-------------------------------------------------------------------------
//      int ArtsIpPathEntry::write(int fd, uint8_t version = 0) const
//.........................................................................
//                                                                         
//-------------------------------------------------------------------------
int ArtsIpPathEntry::write(int fd, uint8_t version = 0) const
{
  int  rc;
  int  bytesWritten = 0;

  if ((rc = ::write(fd,&this->_hopNum,sizeof(this->_hopNum))) !=
      sizeof(this->_hopNum)) {
    return(-1);
  }
  bytesWritten += rc;

  if ((rc = ::write(fd,&this->_ipAddr,sizeof(this->_ipAddr))) !=
      sizeof(this->_ipAddr)) {
    return(-1);
  }
  bytesWritten += rc;

  return(bytesWritten);
}

//------------------------------------------------------------------------
// ostream & operator << (ostream& os, const ArtsIpPathEntry & artsIpPathEntry)
//------------------------------------------------------------------------
ostream & operator << (ostream& os, const ArtsIpPathEntry & artsIpPathEntry)
{
  struct in_addr  inAddr;
  
  inAddr.s_addr = artsIpPathEntry.IpAddr();
  os << "\t\tHopNum: " << setiosflags(ios::left) << setw(3) << dec
     << (int)artsIpPathEntry.HopNum() << " IpAddr: "
     << setiosflags(ios::left) << setw(15) << inet_ntoa(inAddr) << " ("
     << hex << inAddr.s_addr << ")" << endl;
  return(os);
}

uint32_t ArtsIpPathEntry::_numObjects = 0;
