//===========================================================================
//  @(#) $Name: arts++-0-9-b6 $
//  @(#) $Id: ArtsNetMatrixAggregatorMap.cc,v 1.1 1998/10/21 18:35:44 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#include <string>
#include <fstream.h>

#include "ArtsNetMatrixAggregatorMap.hh"

static const string rcsid = "@(#) $Name: arts++-0-9-b6 $ $Id: ArtsNetMatrixAggregatorMap.cc,v 1.1 1998/10/21 18:35:44 dwm Exp $";

//-------------------------------------------------------------------------
//          int ArtsNetMatrixAggregatorMap::Add(const Arts & arts)         
//.........................................................................
//  
//-------------------------------------------------------------------------
int ArtsNetMatrixAggregatorMap::Add(const Arts & arts)
{
  vector<ArtsAttribute>::const_iterator   hostAttribute;
  vector<ArtsAttribute>::const_iterator   ifIndexAttribute;
  ArtsAggregatorMapKey                    netmKey;

  if (arts.Header().Identifier() != artsC_OBJECT_NET)
    return(-1);
  
  hostAttribute = arts.FindHostAttribute();
  if (hostAttribute != arts.Attributes().end()) {
    netmKey.Router(hostAttribute->Host());
  }
  else {
    netmKey.Router(0);
  }

  ifIndexAttribute = arts.FindIfIndexAttribute();
  if (ifIndexAttribute != arts.Attributes().end()) {
    netmKey.IfIndex(ifIndexAttribute->IfIndex());
  }
  else {
    netmKey.IfIndex(0);
  }

  if ((*this).find(netmKey) == (*this).end()) {
    (*this)[netmKey] = new ArtsNetMatrixAggregator(arts);
  }
  else {
    ((*this)[netmKey])->Add(arts);
  }
  
  return(0);
}

//-------------------------------------------------------------------------
//  int ArtsNetMatrixAggregatorMap::AddAllFromFile(const string & fileName, 
//                                                bool quiet)              
//.........................................................................
//  
//-------------------------------------------------------------------------
int ArtsNetMatrixAggregatorMap::AddAllFromFile(const string & fileName,
                                              bool quiet)
{
  ifstream    artsStream(fileName.c_str());
  streampos   streamPosition;
  ArtsHeader  artsHeader;
  Arts       *arts = (Arts *)0;
  int         numAdded = 0;
  
  if (!artsStream) {
    return(-1);
  }

  while (artsStream) {
    //  save our stream position, we might want to back up
    //  and read a whole ARTS object.
    streamPosition = artsStream.tellg();

    //  Read the object header from the stream.  We'll use it to
    //  determine whether or not the object is an AS matrix
    //  object.
    artsHeader.read(artsStream);
    if (! artsStream)
      break;

    if (artsHeader.Identifier() != artsC_OBJECT_AS_MATRIX) {

      //  Not an AS matrix object, skip over it.
      artsStream.seekg(artsHeader.AttrLength() + artsHeader.DataLength(),
                       ios::cur);
    }
    else {
      //  It's an AS matrix object.  Back up to start of header
      //  and read the whole object.
      artsStream.seekg(streamPosition,ios::beg);

      arts = new Arts();
      arts->read(artsStream);

      //  Add the data to the map.
      if (this->Add(*arts) == 0)
        numAdded++;
      
      if (!quiet) {
        cout << ".";
        cout.flush();
      }
      
      delete(arts);
    }
  }
  artsStream.close();
  
  return(numAdded);
}
