//===========================================================================
//  $Name: arts++-0-9-b6 $
//  $Id: ArtsPortChoice.cc,v 1.9 1999/01/25 08:38:15 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

extern "C" {
#include <assert.h>
}

#include <string>

#include "ArtsPrimitive.hh"
#include "ArtsPortChoice.hh"

static const string rcsid = "@(#) $Name: arts++-0-9-b6 $ $Id: ArtsPortChoice.cc,v 1.9 1999/01/25 08:38:15 dwm Exp $";

static ArtsPrimitive g_artsPrimitive;

//-------------------------------------------------------------------------
//                     ArtsPortChoice::ArtsPortChoice()                    
//.........................................................................
//  
//-------------------------------------------------------------------------
ArtsPortChoice::ArtsPortChoice()
{
  #ifdef ARTS_DEBUG_CLASS_ALLOC
    cerr << "ArtsPortChoice() constructor called for "
         << (void *)this << endl;
  #endif
    
  this->_flags = 0x00;
  this->_value.first = 0;
  this->_value.second = 0;

  #ifdef ARTS_DEBUG_CLASS_ALLOC
    cerr << "ArtsPortChoice() constructor done for "
         << (void *)this << endl;
  #endif
}

//-------------------------------------------------------------------------
//              ArtsPortChoice::ArtsPortChoice(uint16_t port)              
//.........................................................................
//  
//-------------------------------------------------------------------------
ArtsPortChoice::ArtsPortChoice(uint16_t port)
{
  #ifdef ARTS_DEBUG_CLASS_ALLOC
    cerr << "ArtsPortChoice(uint16_t port) constructor called for "
         << (void *)this << endl;
  #endif

  this->Value(port);

  #ifdef ARTS_DEBUG_CLASS_ALLOC
    cerr << "ArtsPortChoice(uint16_t port) constructor done for "
         << (void *)this << endl;
  #endif
}

//-------------------------------------------------------------------------
//  ArtsPortChoice::ArtsPortChoice(uint16_t firstPort, uint16_t lastPort)  
//.........................................................................
//  
//-------------------------------------------------------------------------
ArtsPortChoice::ArtsPortChoice(uint16_t firstPort, uint16_t lastPort)
{
  #ifdef ARTS_DEBUG_CLASS_ALLOC
    cerr << "ArtsPortChoice(uint16_t firstPort, uint16_t lastPort)"
         << " constructor called for " << (void *)this << endl;
  #endif

  this->Value(firstPort,lastPort);

  #ifdef ARTS_DEBUG_CLASS_ALLOC
    cerr << "ArtsPortChoice(uint16_t firstPort, uint16_t lastPort)"
         << " constructor done for " << (void *)this << endl;
  #endif
}

//-------------------------------------------------------------------------
//                    ArtsPortChoice::~ArtsPortChoice()                    
//.........................................................................
//  
//-------------------------------------------------------------------------
ArtsPortChoice::~ArtsPortChoice()
{
  #ifdef ARTS_DEBUG_CLASS_ALLOC
    cerr << "ArtsPortChoice destructor called for " << (void *)this << endl;
  #endif
}

//-------------------------------------------------------------------------
//             const value_type & ArtsPortChoice::Value() const            
//.........................................................................
//  
//-------------------------------------------------------------------------
const ArtsPortChoice::value_type & ArtsPortChoice::Value() const
{
  return(this->_value);
}

//-------------------------------------------------------------------------
//              uint16_t ArtsPortChoice::Value(uint16_t port)              
//.........................................................................
//  
//-------------------------------------------------------------------------
uint16_t ArtsPortChoice::Value(uint16_t port)
{
  this->IsRange(false);

  if (port > 0xff)
    this->_flags |= this->k_firstPortLengthMask;
  else
    this->_flags &= (~ this->k_firstPortLengthMask);

  this->_value.first = port;
  
  return(this->_value.first);
}


//-------------------------------------------------------------------------
//                    const ArtsPortChoice::value_type &                   
//                    ArtsPortChoice::Value(uint16_t firstPort,            
//                                          uint16_t lastPort)             
//.........................................................................
//  
//-------------------------------------------------------------------------
const ArtsPortChoice::value_type &
ArtsPortChoice::Value(uint16_t firstPort,
                      uint16_t lastPort)
{
  assert(lastPort >= firstPort);

  this->IsRange(true);

  if (firstPort > 0xff)
    this->_flags |= this->k_firstPortLengthMask;
  else
    this->_flags &= (~ this->k_firstPortLengthMask);
  
  this->_value.first = firstPort;
  
  if (lastPort > 0xff)
    this->_flags |= this->k_lastPortLengthMask;
  else
    this->_flags &= (~ this->k_lastPortLengthMask);
  
  this->_value.second = lastPort;
  
  return(this->_value);
}


//-------------------------------------------------------------------------
//            bool ArtsPortChoice::Matches(uint16_t port) const            
//.........................................................................
//  
//-------------------------------------------------------------------------
bool ArtsPortChoice::Matches(uint16_t port) const
{
  bool  rc = false;
  
  if (this->IsRange()) {
    if (port >= this->_value.first && port <= this->_value.second) {
      rc = true;
    }
  }
  else {
    if (port == this->_value.first) {
      rc = true;
    }
  }
  return(rc);
}

//-------------------------------------------------------------------------
//                 uint32_t ArtsPortChoice::Length() const                 
//.........................................................................
//  
//-------------------------------------------------------------------------
uint32_t ArtsPortChoice::Length() const
{
  uint32_t  length = 0;

  length += sizeof(this->_flags);
  
  length += 1;
  if (this->_flags & this->k_firstPortLengthMask) {
    length += 1;
  }

  if (this->IsRange()) {
    length += 1;
    if (this->_flags & this->k_lastPortLengthMask) {
      length += 1;
    }
  }
  return(length);
}

//-------------------------------------------------------------------------
//                     int ArtsPortChoice::read(int fd)                    
//.........................................................................
//  
//-------------------------------------------------------------------------
int ArtsPortChoice::read(int fd)
{
  int       rc;
  int       bytesRead = 0;
  uint8_t   portLength;
  
  rc = g_artsPrimitive.FdRead(fd,&this->_flags,sizeof(this->_flags));
  if (rc <= 0)
    return(rc);
  bytesRead += rc;

  portLength = 1;
  if (this->_flags & this->k_firstPortLengthMask)
    portLength = 2;
  rc = g_artsPrimitive.ReadUint16(fd,this->_value.first,portLength);
  if (rc < portLength) {
    return(-1);
  }
  bytesRead += rc;

  if (this->IsRange()) {
    portLength = 1;
    if (this->_flags & this->k_lastPortLengthMask)
      portLength = 2;
    rc = g_artsPrimitive.ReadUint16(fd,this->_value.second,portLength);
    if (rc < portLength) {
      return(-1);
    }
    bytesRead += rc;
  }
  return(bytesRead);
}

//-------------------------------------------------------------------------
//               istream & ArtsPortChoice::read(istream & is)              
//.........................................................................
//  
//-------------------------------------------------------------------------
istream & ArtsPortChoice::read(istream & is)
{
  uint8_t   portLength;

  is.read(&this->_flags,sizeof(this->_flags));
  if (!is)
    return(is);
  
  portLength = 1;
  if (this->_flags & this->k_firstPortLengthMask)
    portLength = 2;
  g_artsPrimitive.ReadUint16(is,this->_value.first,portLength);
  if (!is)
    return(is);
  
  if (this->IsRange()) {
    portLength = 1;
    if (this->_flags & this->k_lastPortLengthMask)
      portLength = 2;
    g_artsPrimitive.ReadUint16(is,this->_value.second,portLength);
  }
  return(is);
}

//-------------------------------------------------------------------------
//                 int ArtsPortChoice::write(int fd) const                 
//.........................................................................
//  
//-------------------------------------------------------------------------
int ArtsPortChoice::write(int fd) const
{
  int      rc;
  int      bytesWritten = 0;
  uint8_t  portLength;
  
  rc = g_artsPrimitive.FdWrite(fd,&this->_flags,sizeof(this->_flags));
  if (rc < sizeof(this->_flags)) {
    return(-1);
  }
  bytesWritten += rc;
  
  portLength = 1;
  if (this->_flags & this->k_firstPortLengthMask)
    portLength = 2;
  rc = g_artsPrimitive.WriteUint16(fd,this->_value.first,portLength);
  if (rc < portLength)
    return(-1);
  bytesWritten += rc;

  if (this->IsRange()) {
    portLength = 1;
    if (this->_flags & this->k_lastPortLengthMask)
      portLength = 2;
    rc = g_artsPrimitive.WriteUint16(fd,this->_value.second,portLength);
    if (rc < portLength)
      return(-1);
    bytesWritten += rc;
  }
  return(bytesWritten);
}


//-------------------------------------------------------------------------
//           ostream & ArtsPortChoice::write(ostream & os) const           
//.........................................................................
//  
//-------------------------------------------------------------------------
ostream & ArtsPortChoice::write(ostream & os) const
{
  uint8_t  portLength;
  
  os.write(&this->_flags,sizeof(this->_flags));

  portLength = 1;
  if (this->_flags & this->k_firstPortLengthMask)
    portLength = 2;
  g_artsPrimitive.WriteUint16(os,this->_value.first,portLength);
  if (this->IsRange()) {
    portLength = 1;
    if (this->_flags & this->k_lastPortLengthMask)
      portLength = 2;
    g_artsPrimitive.WriteUint16(os,this->_value.second,portLength);
  }
  return(os);
}

//-------------------------------------------------------------------------
// bool ArtsPortChoice::operator == (const ArtsPortChoice & portChoice) const 
//.........................................................................
//  
//-------------------------------------------------------------------------
bool ArtsPortChoice::operator == (const ArtsPortChoice & portChoice) const
{

  if (this->IsRange() != portChoice.IsRange()) {
    return(false);
  }
  if (this->Value().first != portChoice.Value().first) {
    return(false);
  }
  if (this->IsRange()) {
    if (this->Value().second != portChoice.Value().second) {
      return(false);
    }
  }
  return(true);
}

//-------------------------------------------------------------------------
// bool ArtsPortChoice::operator < (const ArtsPortChoice & portChoice) const 
//.........................................................................
//  
//-------------------------------------------------------------------------
bool ArtsPortChoice::operator < (const ArtsPortChoice & portChoice) const
{
  if (this->Value().first < portChoice.Value().first) {
    return(true);
  }
  if (this->IsRange()) {
    if (this->Value().second < portChoice.Value().second) {
      return(true);
    }
  }

  return(false);
}


//-------------------------------------------------------------------------
//         ostream & operator << (ostream& os,                   
//                                const ArtsPortChoice & portChoice) 
//.........................................................................
//  
//-------------------------------------------------------------------------
ostream & operator << (ostream& os,
                       const ArtsPortChoice & portChoice)
{
  os << portChoice.Value().first;
  if (portChoice.IsRange())
    os << "-" << portChoice.Value().second;
  return(os);
}
