//===========================================================================
//  @(#) $Name: arts++-0-9-b6 $
//  @(#) $Id: ArtsPortTableData.cc,v 1.4 1999/01/25 08:38:20 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#include <string>
#include <algo.h>

#include "Arts.hh"
#include "ArtsPrimitive.hh"
#include "ArtsPortTableData.hh"

static const string rcsid = "@(#) $Name: arts++-0-9-b6 $ $Id: ArtsPortTableData.cc,v 1.4 1999/01/25 08:38:20 dwm Exp $";

static ArtsPrimitive  artsPrimitive;

template class vector<ArtsPortTableEntry>;

//-------------------------------------------------------------------------
//                  ArtsPortTableData::ArtsPortTableData()                 
//.........................................................................
//  
//-------------------------------------------------------------------------
ArtsPortTableData::ArtsPortTableData()
{
  this->_sampleInterval = 0;
  this->_totpkts = 0;
  this->_totbytes = 0;
  this->_length = 0;

  #ifdef ARTS_DEBUG_CLASS_ALLOC
    ++this->_numObjects;
  #endif
}

//-------------------------------------------------------------------------
//                 ArtsPortTableData::~ArtsPortTableData()                 
//.........................................................................
//  
//-------------------------------------------------------------------------
ArtsPortTableData::~ArtsPortTableData()
{
  #ifdef ARTS_DEBUG_CLASS_ALLOC
    --this->_numObjects;
  #endif
}

//-------------------------------------------------------------------------
//           uint16_t ArtsPortTableData::SampleInterval() const           
//.........................................................................
//  
//-------------------------------------------------------------------------
uint16_t ArtsPortTableData::SampleInterval() const
{
  return(this->_sampleInterval);
}

//-------------------------------------------------------------------------
//   uint16_t ArtsPortTableData::SampleInterval(uint16_t sampleInterval)
//.........................................................................
//  
//-------------------------------------------------------------------------
uint16_t ArtsPortTableData::SampleInterval(uint16_t sampleInterval)
{
  this->_sampleInterval = sampleInterval;
  return(this->_sampleInterval);
}

//-------------------------------------------------------------------------
//              uint64_t ArtsPortTableData::TotalPkts() const              
//.........................................................................
//  
//-------------------------------------------------------------------------
uint64_t ArtsPortTableData::TotalPkts() const
{
  return(this->_totpkts);
}

//-------------------------------------------------------------------------
//        uint64_t ArtsPortTableData::TotalPkts(uint64_t totalPkts)
//.........................................................................
//  
//-------------------------------------------------------------------------
uint64_t ArtsPortTableData::TotalPkts(uint64_t totalPkts)
{
  this->_totpkts = totalPkts;
  return(this->_totpkts);
}

//-------------------------------------------------------------------------
//             uint64_t ArtsPortTableData::TotalBytes() const
//.........................................................................
//  
//-------------------------------------------------------------------------
uint64_t ArtsPortTableData::TotalBytes() const
{
  return(this->_totbytes);
}

//-------------------------------------------------------------------------
//       uint64_t ArtsPortTableData::TotalBytes(uint64_t totalBytes)
//.........................................................................
//  
//-------------------------------------------------------------------------
uint64_t ArtsPortTableData::TotalBytes(uint64_t totalBytes)
{
  this->_totbytes = totalBytes;
  return(this->_totbytes);
}

//-------------------------------------------------------------------------
//   vector<ArtsPortTableEntry> & ArtsPortTableData::PortEntries() const
//.........................................................................
//  
//-------------------------------------------------------------------------
vector<ArtsPortTableEntry> & ArtsPortTableData::PortEntries() const
{
  return(this->_portEntries);
}

//-------------------------------------------------------------------------
//               void ArtsPortTableData::SortEntriesByBytes()              
//.........................................................................
//  
//-------------------------------------------------------------------------
void ArtsPortTableData::SortEntriesByBytes()
{
  sort(this->_portEntries.begin(),this->_portEntries.end(),
       ArtsPortEntryGreaterBytes());
  
  return;
}

//-------------------------------------------------------------------------
//               void ArtsPortTableData::SortEntriesByPkts()              
//.........................................................................
//  
//-------------------------------------------------------------------------
void ArtsPortTableData::SortEntriesByPkts()
{
  sort(this->_portEntries.begin(),this->_portEntries.end(),
       ArtsPortEntryGreaterPkts());
  
  return;
}

//-------------------------------------------------------------------------
//  uint32_t ArtsPortTableData::ComputeLength(uint8_t version = 0) const
//.........................................................................
//  
//-------------------------------------------------------------------------
uint32_t ArtsPortTableData::ComputeLength(uint8_t version = 0) const
{
  this->_length = 0;
  
  this->_length += sizeof(_sampleInterval);
  this->_length += sizeof(_totpkts);
  this->_length += sizeof(_totbytes);
  this->_length += sizeof(uint32_t);  // number of port entries

  vector<ArtsPortTableEntry>::const_iterator  portEntry;

  for (portEntry = this->_portEntries.begin();
       portEntry != this->_portEntries.end(); portEntry++) {
    this->_length += portEntry->Length(version);
  }
  return(this->_length);
}

//-------------------------------------------------------------------------
//     uint32_t ArtsPortTableData::Length(uint8_t version = 0) const
//.........................................................................
//  
//-------------------------------------------------------------------------
uint32_t ArtsPortTableData::Length(uint8_t version = 0) const
{
  this->ComputeLength(version);
  return(this->_length);
}

//-------------------------------------------------------------------------
//   istream& ArtsPortTableData::read(istream& is, uint8_t version = 0)
//.........................................................................
//  
//-------------------------------------------------------------------------
istream& ArtsPortTableData::read(istream& is, uint8_t version = 0)
{
  uint32_t            numPorts;
  uint32_t            portNum;
  ArtsPortTableEntry  portEntry;
  
  artsPrimitive.ReadUint16(is,this->_sampleInterval,2);
  artsPrimitive.ReadUint64(is,this->_totpkts,8);
  artsPrimitive.ReadUint64(is,this->_totbytes,8);
  artsPrimitive.ReadUint32(is,numPorts,4);
  this->_portEntries.reserve(numPorts);
  for (portNum = 0; portNum < numPorts; portNum++) {
    portEntry.read(is,version);
    this->_portEntries.push_back(portEntry);
  }
  return(is);
}

//-------------------------------------------------------------------------
//         int ArtsPortTableData::read(int fd, uint8_t version = 0)
//.........................................................................
//  
//-------------------------------------------------------------------------
int ArtsPortTableData::read(int fd, uint8_t version = 0)
{
  uint32_t            numPorts;
  uint32_t            portNum;
  ArtsPortTableEntry  portEntry;
  int                 rc;
  int                 bytesRead = 0;

  rc = artsPrimitive.ReadUint16(fd,this->_sampleInterval,2);
  if (rc < 2)
    return(-1);
  bytesRead += rc;
  rc = artsPrimitive.ReadUint64(fd,this->_totpkts,8);
  if (rc < 8)
    return(-1);
  bytesRead += rc;
  rc = artsPrimitive.ReadUint64(fd,this->_totbytes,8);
  if (rc < 8)
    return(-1);
  bytesRead += rc;
  rc = artsPrimitive.ReadUint32(fd,numPorts,4);
  if (rc < 4)
    return(-1);
  bytesRead += rc;
  for (portNum = 0; portNum < numPorts; portNum++) {
    rc = portEntry.read(fd,version);
    if (rc < 0)
      return(rc);
    bytesRead += rc;
    this->_portEntries.push_back(portEntry);
  }

  return(bytesRead);
}

//-------------------------------------------------------------------------
//      ostream& ArtsPortTableData::write(ostream& os,
//                                        uint8_t version = 0) const
//.........................................................................
//  
//-------------------------------------------------------------------------
ostream& ArtsPortTableData::write(ostream& os,
                                  uint8_t version = 0) const
{
  uint32_t                                    numPorts;
  vector<ArtsPortTableEntry>::const_iterator  portEntry;
  
  artsPrimitive.WriteUint16(os,this->_sampleInterval,2);
  artsPrimitive.WriteUint64(os,this->_totpkts,8);
  artsPrimitive.WriteUint64(os,this->_totbytes,8);
  numPorts = this->_portEntries.size();
  artsPrimitive.WriteUint32(os,numPorts,4);

  for (portEntry = this->_portEntries.begin();
       portEntry != this->_portEntries.end(); portEntry++) {
    portEntry->write(os,version);
  }

  return(os);
}

//-------------------------------------------------------------------------
//     int ArtsPortTableData::write(int fd, uint8_t version = 0) const
//.........................................................................
//  
//-------------------------------------------------------------------------
int ArtsPortTableData::write(int fd, uint8_t version = 0) const
{
  uint32_t      numPorts;
  int           rc;
  int           bytesWritten = 0;
  
  rc = artsPrimitive.WriteUint16(fd,this->_sampleInterval,2);
  if (rc < 2)
    return(-1);
  bytesWritten += rc;
  
  rc = artsPrimitive.WriteUint64(fd,this->_totpkts,8);
  if (rc < 8)
    return(-1);
  bytesWritten += rc;

  rc = artsPrimitive.WriteUint64(fd,this->_totbytes,8);
  if (rc < 8)
    return(-1);
  bytesWritten += rc;

  numPorts = this->_portEntries.size();
  artsPrimitive.WriteUint32(fd,numPorts,4);
  if (rc < 4)
    return(-1);
  bytesWritten += rc;

  vector<ArtsPortTableEntry>::const_iterator  portEntry;
  
  for (portEntry = this->_portEntries.begin();
       portEntry != this->_portEntries.end(); portEntry++) {
    rc = portEntry->write(fd,version);
    if (rc < 0)
      return(rc);
    bytesWritten += rc;
  }

  return(bytesWritten);
}

//-------------------------------------------------------------------------
// ArtsPortTableData &                           
// ArtsPortTableData::operator = (const ArtsPortTableData & artsPortTableData) 
//.........................................................................
//  
//-------------------------------------------------------------------------
ArtsPortTableData &
ArtsPortTableData::operator = (const ArtsPortTableData & artsPortTableData)
{
  this->_sampleInterval = artsPortTableData.SampleInterval();
  this->_totpkts = artsPortTableData.TotalPkts();
  this->_totbytes = artsPortTableData.TotalBytes();
  this->_portEntries = artsPortTableData.PortEntries();
  this->_length = artsPortTableData.Length();
  return(*this);
}

//-------------------------------------------------------------------------
//   ostream& operator << (ostream& os,                   
//                         const ArtsPortTableData & artsPortTableData) 
//.........................................................................
//  
//-------------------------------------------------------------------------
ostream& operator << (ostream& os,
                      const ArtsPortTableData & artsPortTableData)
{
  os << "PORT OBJECT DATA" << endl;
  
  os << "\tsample_interval: " << artsPortTableData.SampleInterval() << endl;
  os << "\tcount: " << artsPortTableData.PortEntries().size() << endl;
  os << "\ttotpkts: " << artsPortTableData.TotalPkts() << endl;
  os << "\ttotbytes: " << artsPortTableData.TotalBytes() << endl;
  
  vector<ArtsPortTableEntry>::const_iterator  portEntry;
  for (portEntry = artsPortTableData.PortEntries().begin();
       portEntry != artsPortTableData.PortEntries().end(); portEntry++) {
    os << *portEntry;
  }
  return(os);
}

uint32_t ArtsPortTableData::_numObjects = 0;
